@@ -30,6 +30,7 @@ const (
 
 // See https://cloud.google.com/storage/docs/using-encryption-keys#generating_your_own_encryption_key
 const encryptionKey = "yRyCOikXi1ZDNE0xN3yiFsJjg7LGimoLrGFcLZgQoVk="
+const encryptionKey2 = "cRKXxV+HVAITvGlqxLJL8hZ95VTFHT2djkoRQjBpQls="
 
 // KMS key ring name and key name are hardcoded here and re-used because key rings (and keys) cannot be deleted
 // Test code asserts their presence and creates them if they're absent. They're not deleted at the end of tests.
@@ -42,59 +43,112 @@ const (
 
 var keyRingLocation = os.Getenv("GOOGLE_REGION")
 
-func TestStateFile(t *testing.T) {
-	t.Parallel()
+func preCheckTestAcc(t *testing.T) {
+	if v := os.Getenv("TF_ACC"); v == "" {
+		t.Skip("TF_ACC must be set to run acceptance tests, as they provision real resources")
+	}
+}
 
-	cases := []struct {
-		prefix        string
-		name          string
-		wantStateFile string
-		wantLockFile  string
+func preCheckEnvironmentVariables(t *testing.T) {
+
+	// TODO - implement a separate function specific to credentials which will iterate through a list of ENV names and return first value found.
+
+	credentials := []string{
+		"GOOGLE_BACKEND_CREDENTIALS",
+		"GOOGLE_CREDENTIALS",
+	}
+	credsFound := false
+	for _, name := range credentials {
+		v := os.Getenv(name)
+		if v != "" {
+			credsFound = true
+			break
+		}
+	}
+	if !credsFound {
+		// Skipping tests because hashicorp/terraform repo doesn't have credentials set up.
+		// In future we should enable tests to run automatically, and make this code fail when no creds are set.
+		t.Skip("credentials need to be provided via GOOGLE_BACKEND_CREDENTIALS or GOOGLE_CREDENTIALS environment variable but neither is set")
+	}
+}
+
+func TestAccBackendConfig_credentials(t *testing.T) {
+	// Cannot use t.Parallel() due to t.Setenv
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
+
+	credentials := os.Getenv("GOOGLE_BACKEND_CREDENTIALS")
+	if credentials == "" {
+		credentials = os.Getenv("GOOGLE_CREDENTIALS")
+	}
+	if credentials == "" {
+		t.Fatalf("unable to access credentials from the test environment")
+	}
+
+	t.Setenv("GOOGLE_BACKEND_CREDENTIALS", "") // unset value
+	t.Setenv("GOOGLE_CREDENTIALS", "")         // unset value
+
+	cases := map[string]struct {
+		config map[string]interface{}
+		envs   map[string]string
+		want   string
 	}{
-		{"state", "default", "state/default.tfstate", "state/default.tflock"},
-		{"state", "test", "state/test.tfstate", "state/test.tflock"},
-		{"state", "test", "state/test.tfstate", "state/test.tflock"},
-		{"state", "test", "state/test.tfstate", "state/test.tflock"},
+		"empty credentials in config doesn't affect use of GOOGLE_BACKEND_CREDENTIALS": {
+			config: map[string]interface{}{
+				"bucket":      "tf-test-testaccbackendconfig_credentials_1",
+				"credentials": "",
+			},
+			envs: map[string]string{
+				"GOOGLE_BACKEND_CREDENTIALS": credentials,
+			},
+		},
+		"empty credentials in config doesn't affect use of GOOGLE_CREDENTIALS": {
+			config: map[string]interface{}{
+				"bucket":      "tf-test-testaccbackendconfig_credentials_2",
+				"credentials": "",
+			},
+			envs: map[string]string{
+				"GOOGLE_CREDENTIALS": credentials,
+			},
+		},
+		"credentials in config that isn't an empty string takes precedence over credentials from the environment": {
+			config: map[string]interface{}{
+				"bucket":      "tf-test-testaccbackendconfig_credentials_3",
+				"credentials": credentials,
+			},
+			envs: map[string]string{
+				"GOOGLE_CREDENTIALS": "foobar", // Would cause an error if used
+			},
+		},
 	}
-	for _, c := range cases {
-		b := &Backend{
-			prefix: c.prefix,
-		}
+	for name, tc := range cases {
+		t.Run(name, func(t *testing.T) {
+			for k, v := range tc.envs {
+				t.Setenv(k, v)
+			}
 
-		if got := b.stateFile(c.name); got != c.wantStateFile {
-			t.Errorf("stateFile(%q) = %q, want %q", c.name, got, c.wantStateFile)
-		}
+			be0 := setupBackend(t, tc.config)
+			defer teardownBackend(t, be0, noPrefix)
 
-		if got := b.lockFile(c.name); got != c.wantLockFile {
-			t.Errorf("lockFile(%q) = %q, want %q", c.name, got, c.wantLockFile)
-		}
+			be1 := setupBackend(t, tc.config)
+
+			backend.TestBackendStates(t, be0)
+			backend.TestBackendStateLocks(t, be0, be1)
+		})
 	}
+
 }
 
-func TestRemoteClient(t *testing.T) {
+func TestAccRemoteClient(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	bucket := bucketName(t)
-	be := setupBackend(t, bucket, noPrefix, noEncryptionKey, noKmsKeyName)
-	defer teardownBackend(t, be, noPrefix)
-
-	ss, err := be.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatalf("be.StateMgr(%q) = %v", backend.DefaultStateName, err)
+	config := map[string]interface{}{
+		"bucket": bucket,
 	}
-
-	rs, ok := ss.(*remote.State)
-	if !ok {
-		t.Fatalf("be.StateMgr(): got a %T, want a *remote.State", ss)
-	}
-
-	remote.TestClient(t, rs.Client)
-}
-func TestRemoteClientWithEncryption(t *testing.T) {
-	t.Parallel()
-
-	bucket := bucketName(t)
-	be := setupBackend(t, bucket, noPrefix, encryptionKey, noKmsKeyName)
+	be := setupBackend(t, config)
 	defer teardownBackend(t, be, noPrefix)
 
 	ss, err := be.StateMgr(backend.DefaultStateName)
@@ -110,11 +164,42 @@ func TestRemoteClientWithEncryption(t *testing.T) {
 	remote.TestClient(t, rs.Client)
 }
 
-func TestRemoteLocks(t *testing.T) {
+func TestAccRemoteClientWithEncryption(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	bucket := bucketName(t)
-	be := setupBackend(t, bucket, noPrefix, noEncryptionKey, noKmsKeyName)
+	config := map[string]interface{}{
+		"bucket":         bucket,
+		"encryption_key": encryptionKey,
+	}
+	be := setupBackend(t, config)
+	defer teardownBackend(t, be, noPrefix)
+
+	ss, err := be.StateMgr(backend.DefaultStateName)
+	if err != nil {
+		t.Fatalf("be.StateMgr(%q) = %v", backend.DefaultStateName, err)
+	}
+
+	rs, ok := ss.(*remote.State)
+	if !ok {
+		t.Fatalf("be.StateMgr(): got a %T, want a *remote.State", ss)
+	}
+
+	remote.TestClient(t, rs.Client)
+}
+
+func TestAccRemoteLocks(t *testing.T) {
+	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
+
+	bucket := bucketName(t)
+	config := map[string]interface{}{
+		"bucket": bucket,
+	}
+	be := setupBackend(t, config)
 	defer teardownBackend(t, be, noPrefix)
 
 	remoteClient := func() (remote.Client, error) {
@@ -143,51 +228,73 @@ func TestRemoteLocks(t *testing.T) {
 	remote.TestRemoteLocks(t, c0, c1)
 }
 
-func TestBackend(t *testing.T) {
+func TestAccBackend(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	bucket := bucketName(t)
 
-	be0 := setupBackend(t, bucket, noPrefix, noEncryptionKey, noKmsKeyName)
+	config := map[string]interface{}{
+		"bucket": bucket,
+	}
+	be0 := setupBackend(t, config)
 	defer teardownBackend(t, be0, noPrefix)
 
-	be1 := setupBackend(t, bucket, noPrefix, noEncryptionKey, noKmsKeyName)
+	be1 := setupBackend(t, config)
 
 	backend.TestBackendStates(t, be0)
 	backend.TestBackendStateLocks(t, be0, be1)
 	backend.TestBackendStateForceUnlock(t, be0, be1)
 }
 
-func TestBackendWithPrefix(t *testing.T) {
+func TestAccBackendWithPrefix(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	prefix := "test/prefix"
 	bucket := bucketName(t)
 
-	be0 := setupBackend(t, bucket, prefix, noEncryptionKey, noKmsKeyName)
+	config := map[string]interface{}{
+		"bucket":         bucket,
+		"prefix":         prefix,
+		"encryption_key": encryptionKey,
+	}
+	be0 := setupBackend(t, config)
 	defer teardownBackend(t, be0, prefix)
 
-	be1 := setupBackend(t, bucket, prefix+"/", noEncryptionKey, noKmsKeyName)
+	config["prefix"] = prefix + "/"
+	be1 := setupBackend(t, config)
 
 	backend.TestBackendStates(t, be0)
 	backend.TestBackendStateLocks(t, be0, be1)
 }
-func TestBackendWithCustomerSuppliedEncryption(t *testing.T) {
+
+func TestAccBackendWithCustomerSuppliedEncryption(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	bucket := bucketName(t)
 
-	be0 := setupBackend(t, bucket, noPrefix, encryptionKey, noKmsKeyName)
+	config := map[string]interface{}{
+		"bucket":         bucket,
+		"encryption_key": encryptionKey,
+	}
+	be0 := setupBackend(t, config)
 	defer teardownBackend(t, be0, noPrefix)
 
-	be1 := setupBackend(t, bucket, noPrefix, encryptionKey, noKmsKeyName)
+	be1 := setupBackend(t, config)
 
 	backend.TestBackendStates(t, be0)
 	backend.TestBackendStateLocks(t, be0, be1)
 }
 
-func TestBackendWithCustomerManagedKMSEncryption(t *testing.T) {
+func TestAccBackendWithCustomerManagedKMSEncryption(t *testing.T) {
 	t.Parallel()
+	preCheckTestAcc(t)
+	preCheckEnvironmentVariables(t)
 
 	projectID := os.Getenv("GOOGLE_PROJECT")
 	bucket := bucketName(t)
@@ -202,18 +309,25 @@ func TestBackendWithCustomerManagedKMSEncryption(t *testing.T) {
 
 	kmsName := setupKmsKey(t, kmsDetails)
 
-	be0 := setupBackend(t, bucket, noPrefix, noEncryptionKey, kmsName)
+	config := map[string]interface{}{
+		"bucket":             bucket,
+		"prefix":             noPrefix,
+		"kms_encryption_key": kmsName,
+	}
+
+	be0 := setupBackend(t, config)
 	defer teardownBackend(t, be0, noPrefix)
 
-	be1 := setupBackend(t, bucket, noPrefix, noEncryptionKey, kmsName)
+	be1 := setupBackend(t, config)
 
 	backend.TestBackendStates(t, be0)
 	backend.TestBackendStateLocks(t, be0, be1)
 }
 
 // setupBackend returns a new GCS backend.
-func setupBackend(t *testing.T, bucket, prefix, key, kmsName string) backend.Backend {
+func setupBackend(t *testing.T, config map[string]interface{}) backend.Backend {
 	t.Helper()
+	ctx := context.Background()
 
 	projectID := os.Getenv("GOOGLE_PROJECT")
 	if projectID == "" || os.Getenv("TF_ACC") == "" {
@@ -222,25 +336,12 @@ func setupBackend(t *testing.T, bucket, prefix, key, kmsName string) backend.Bac
 			"the TF_ACC and GOOGLE_PROJECT environment variables are set.")
 	}
 
-	config := map[string]interface{}{
-		"bucket": bucket,
-		"prefix": prefix,
-	}
-	// Only add encryption keys to config if non-zero value set
-	// If not set here, default values are supplied in `TestBackendConfig` by `PrepareConfig` function call
-	if len(key) > 0 {
-		config["encryption_key"] = key
-	}
-	if len(kmsName) > 0 {
-		config["kms_encryption_key"] = kmsName
-	}
-
 	b := backend.TestBackendConfig(t, New(), backend.TestWrapConfig(config))
 	be := b.(*Backend)
 
 	// create the bucket if it doesn't exist
-	bkt := be.storageClient.Bucket(bucket)
-	_, err := bkt.Attrs(be.storageContext)
+	bkt := be.storageClient.Bucket(config["bucket"].(string))
+	_, err := bkt.Attrs(ctx)
 	if err != nil {
 		if err != storage.ErrBucketNotExist {
 			t.Fatal(err)
@@ -249,7 +350,7 @@ func setupBackend(t *testing.T, bucket, prefix, key, kmsName string) backend.Bac
 		attrs := &storage.BucketAttrs{
 			Location: os.Getenv("GOOGLE_REGION"),
 		}
-		err := bkt.Create(be.storageContext, projectID, attrs)
+		err := bkt.Create(ctx, projectID, attrs)
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -379,7 +480,7 @@ func teardownBackend(t *testing.T, be backend.Backend, prefix string) {
 	if !ok {
 		t.Fatalf("be is a %T, want a *gcsBackend", be)
 	}
-	ctx := gcsBE.storageContext
+	ctx := context.Background()
 
 	bucket := gcsBE.storageClient.Bucket(gcsBE.bucketName)
 	objs := bucket.Objects(ctx, nil)
@@ -429,7 +530,7 @@ func testGetClientOptions(t *testing.T) ([]option.ClientOption, error) {
 	var opts []option.ClientOption
 	var credOptions []option.ClientOption
 
-	contents, err := backend.ReadPathOrContents(creds)
+	contents, err := readPathOrContents(creds)
 	if err != nil {
 		return nil, fmt.Errorf("error loading credentials: %s", err)
 	}
