@@ -379,6 +379,9 @@ Options:
   -parallelism=n         Limit the number of parallel resource operations.
                          Defaults to 10.
 
+  -replace=resource      Terraform will plan to replace this resource instance
+                         instead of doing an update or no-op action. 
+
   -state=path            Path to read and save state (unless state-out
                          is specified). Defaults to "terraform.tfstate".
 
