@@ -10,6 +10,7 @@ import (
 	"os"
 	"path"
 	"regexp"
+	"runtime"
 	"strings"
 	"testing"
 
@@ -35,11 +36,20 @@ func TestTest_Runs(t *testing.T) {
 		code                  int
 		initCode              int
 		skip                  bool
+		description           string
 	}{
 		"simple_pass": {
 			expectedOut: []string{"1 passed, 0 failed."},
 			code:        0,
 		},
+		"top-dir-only-test-files": {
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+		},
+		"top-dir-only-nested-test-files": {
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+		},
 		"simple_pass_nested": {
 			expectedOut: []string{"1 passed, 0 failed."},
 			code:        0,
@@ -54,6 +64,13 @@ func TestTest_Runs(t *testing.T) {
 			expectedOut: []string{"1 passed, 0 failed."},
 			code:        0,
 		},
+		"simple_pass_cmd_parallel": {
+			override:    "simple_pass",
+			args:        []string{"-parallelism", "1"},
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+			description: "simple_pass with parallelism set to 1",
+		},
 		"simple_pass_very_nested_alternate": {
 			override:    "simple_pass_very_nested",
 			args:        []string{"-test-directory", "./tests/subdir"},
@@ -96,10 +113,6 @@ func TestTest_Runs(t *testing.T) {
 			expectedOut: []string{"1 passed, 0 failed."},
 			code:        0,
 		},
-		"expect_failures_outputs": {
-			expectedOut: []string{"1 passed, 0 failed."},
-			code:        0,
-		},
 		"expect_failures_resources": {
 			expectedOut: []string{"1 passed, 0 failed."},
 			code:        0,
@@ -114,6 +127,12 @@ func TestTest_Runs(t *testing.T) {
 			expectedOut: []string{"1 passed, 0 failed"},
 			code:        0,
 		},
+		"no_state": {
+			expectedOut: []string{"0 passed, 1 failed"},
+			expectedErr: []string{"No value for required variable"},
+			description: "the run apply fails, causing it to produce a nil state.",
+			code:        1,
+		},
 		"variables": {
 			expectedOut: []string{"2 passed, 0 failed"},
 			code:        0,
@@ -127,8 +146,9 @@ func TestTest_Runs(t *testing.T) {
 		},
 		"simple_fail": {
 			expectedOut: []string{"0 passed, 1 failed."},
-			expectedErr: []string{"invalid value"},
-			code:        1,
+			expectedErr: []string{"invalid value", `│ - "bar"
+    │ + "zap"`},
+			code: 1,
 		},
 		"custom_condition_checks": {
 			expectedOut: []string{"0 passed, 1 failed."},
@@ -163,7 +183,7 @@ func TestTest_Runs(t *testing.T) {
 			code:        0,
 		},
 		"shared_state": {
-			expectedOut: []string{"2 passed, 0 failed."},
+			expectedOut: []string{"8 passed, 0 failed."},
 			code:        0,
 		},
 		"shared_state_object": {
@@ -193,10 +213,10 @@ func TestTest_Runs(t *testing.T) {
 			code:        0,
 		},
 		"destroy_fail": {
-			expectedOut:           []string{"1 passed, 0 failed."},
+			expectedOut:           []string{"3 passed, 0 failed."},
 			expectedErr:           []string{`Terraform left the following resources in state`},
 			code:                  1,
-			expectedResourceCount: 1,
+			expectedResourceCount: 4,
 		},
 		"default_optional_values": {
 			expectedOut: []string{"4 passed, 0 failed."},
@@ -282,8 +302,10 @@ func TestTest_Runs(t *testing.T) {
 		},
 		"ephemeral_input_with_error": {
 			expectedOut: []string{"Error message refers to ephemeral values", "1 passed, 1 failed."},
-			expectedErr: []string{"Test assertion failed", "has an ephemeral value"},
-			code:        1,
+			expectedErr: []string{"Test assertion failed",
+				`│ - "(ephemeral value)"
+    │ + "bar"`},
+			code: 1,
 		},
 		"ephemeral_resource": {
 			expectedOut: []string{"0 passed, 1 failed."},
@@ -300,6 +322,18 @@ func TestTest_Runs(t *testing.T) {
 			expectedErr: []string{"Cannot apply non-applyable plan"},
 			code:        1,
 		},
+		"write-only-attributes": {
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+		},
+		"write-only-attributes-mocked": {
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+		},
+		"write-only-attributes-overridden": {
+			expectedOut: []string{"1 passed, 0 failed."},
+			code:        0,
+		},
 	}
 	for name, tc := range tcs {
 		t.Run(name, func(t *testing.T) {
@@ -394,7 +428,7 @@ func TestTest_Runs(t *testing.T) {
 				Meta: meta,
 			}
 
-			code := c.Run(tc.args)
+			code := c.Run(append(tc.args, "-no-color"))
 			output := done(t)
 
 			if code != tc.code {
@@ -458,6 +492,327 @@ func TestTest_Interrupt(t *testing.T) {
 	}
 }
 
+func TestTest_DestroyFail(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "destroy_fail")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+	providerSource, close := newMockProviderSource(t, map[string][]string{
+		"test": {"1.0.0"},
+	})
+	defer close()
+
+	view, done := testView(t)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		View:             view,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{Meta: meta}
+	if code := init.Run(nil); code != 0 {
+		output := done(t)
+		t.Fatalf("expected status code %d but got %d: %s", 0, code, output.All())
+	}
+
+	interrupt := make(chan struct{})
+	provider.Interrupt = interrupt
+	view, done = testView(t)
+
+	c := &TestCommand{
+		Meta: Meta{
+			testingOverrides: metaOverridesForProvider(provider.Provider),
+			View:             view,
+			ShutdownCh:       interrupt,
+		},
+	}
+
+	c.Run([]string{"-no-color"})
+	output := done(t)
+	err := output.Stderr()
+
+	cleanupMessage := `main.tftest.hcl... in progress
+  run "setup"... pass
+  run "single"... pass
+  run "double"... pass
+main.tftest.hcl... tearing down
+main.tftest.hcl... fail
+
+Failure! 3 passed, 0 failed.
+`
+
+	cleanupErr := `Terraform encountered an error destroying resources created while executing
+main.tftest.hcl/double.
+
+Error: Failed to destroy resource
+
+destroy_fail is set to true
+
+Error: Failed to destroy resource
+
+destroy_fail is set to true
+
+Terraform left the following resources in state after executing
+main.tftest.hcl/double, and they need to be cleaned up manually:
+  - test_resource.another
+  - test_resource.resource
+Terraform encountered an error destroying resources created while executing
+main.tftest.hcl/single.
+
+Error: Failed to destroy resource
+
+destroy_fail is set to true
+
+Error: Failed to destroy resource
+
+destroy_fail is set to true
+
+Terraform left the following resources in state after executing
+main.tftest.hcl/single, and they need to be cleaned up manually:
+  - test_resource.another
+  - test_resource.resource
+`
+
+	// It's really important that the above message is printed, so we're testing
+	// for it specifically and making sure it contains all the resources.
+	if diff := cmp.Diff(cleanupErr, err); diff != "" {
+		t.Errorf("expected err to be %s\n\nbut got %s\n\n diff:%s\n", cleanupErr, err, diff)
+	}
+	if diff := cmp.Diff(cleanupMessage, output.Stdout()); diff != "" {
+		t.Errorf("expected output to be %s\n\nbut got %s\n\n diff:%s\n", cleanupMessage, output.Stdout(), diff)
+	}
+
+	// This time the test command shouldn't have cleaned up the resource because
+	// the destroy failed.
+	if provider.ResourceCount() != 4 {
+		t.Errorf("should not have deleted all resources on completion but only has %v", provider.ResourceString())
+	}
+}
+
+func TestTest_SharedState_Order(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "shared_state")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+	providerSource, close := newMockProviderSource(t, map[string][]string{
+		"test": {"1.0.0"},
+	})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		Ui:               ui,
+		View:             view,
+		Streams:          streams,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	if code := init.Run(nil); code != 0 {
+		output := done(t)
+		t.Fatalf("expected status code %d but got %d: %s", 9, code, output.All())
+	}
+
+	c := &TestCommand{
+		Meta: meta,
+	}
+
+	c.Run(nil)
+	output := done(t).All()
+
+	// Split the log into lines
+	lines := strings.Split(output, "\n")
+
+	var arr []string
+	for _, line := range lines {
+		if strings.Contains(line, "run \"") && strings.Contains(line, "\x1b[32mpass") {
+			arr = append(arr, line)
+		}
+	}
+
+	// Ensure the order of the tests is correct. Even though they share no state,
+	// the order should be sequential.
+	expectedOrder := []string{
+		// main.tftest.hcl
+		"run \"setup\"",
+		"run \"test\"",
+
+		// no-shared-state.tftest.hcl
+		"run \"setup\"",
+		"run \"test_a\"",
+		"run \"test_b\"",
+		"run \"test_c\"",
+		"run \"test_d\"",
+		"run \"test_e\"",
+	}
+
+	for i, line := range expectedOrder {
+		if !strings.Contains(arr[i], line) {
+			t.Errorf("unexpected test order: expected %q, got %q", line, arr[i])
+		}
+	}
+}
+
+func TestTest_Parallel_Divided_Order(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "parallel_divided")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+	providerSource, close := newMockProviderSource(t, map[string][]string{
+		"test": {"1.0.0"},
+	})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		Ui:               ui,
+		View:             view,
+		Streams:          streams,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	if code := init.Run(nil); code != 0 {
+		output := done(t)
+		t.Fatalf("expected status code %d but got %d: %s", 9, code, output.All())
+	}
+
+	c := &TestCommand{
+		Meta: meta,
+	}
+
+	c.Run(nil)
+	output := done(t).All()
+
+	// Split the log into lines
+	lines := strings.Split(output, "\n")
+
+	// Find the positions of the tests in the log output
+	var mainFirstIndex, mainSecondIndex, mainThirdIndex, mainFourthIndex, mainFifthIndex, mainSixthIndex int
+	for i, line := range lines {
+		if strings.Contains(line, "run \"main_first\"") {
+			mainFirstIndex = i
+		} else if strings.Contains(line, "run \"main_second\"") {
+			mainSecondIndex = i
+		} else if strings.Contains(line, "run \"main_third\"") {
+			mainThirdIndex = i
+		} else if strings.Contains(line, "run \"main_fourth\"") {
+			mainFourthIndex = i
+		} else if strings.Contains(line, "run \"main_fifth\"") {
+			mainFifthIndex = i
+		} else if strings.Contains(line, "run \"main_sixth\"") {
+			mainSixthIndex = i
+		}
+	}
+	if mainFirstIndex == 0 || mainSecondIndex == 0 || mainThirdIndex == 0 || mainFourthIndex == 0 || mainFifthIndex == 0 || mainSixthIndex == 0 {
+		t.Fatalf("one or more tests not found in the log output")
+	}
+
+	// Ensure the order of the tests is correct. The runs before main_fourth can execute in parallel.
+	if mainFirstIndex > mainFourthIndex || mainSecondIndex > mainFourthIndex || mainThirdIndex > mainFourthIndex {
+		t.Errorf("main_first, main_second, or main_third appears after main_fourth in the log output")
+	}
+
+	// Ensure main_fifth and main_sixth do not execute before main_fourth
+	if mainFifthIndex < mainFourthIndex {
+		t.Errorf("main_fifth appears before main_fourth in the log output")
+	}
+	if mainSixthIndex < mainFourthIndex {
+		t.Errorf("main_sixth appears before main_fourth in the log output")
+	}
+}
+
+func TestTest_Parallel(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "parallel")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+	providerSource, close := newMockProviderSource(t, map[string][]string{
+		"test": {"1.0.0"},
+	})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		Ui:               ui,
+		View:             view,
+		Streams:          streams,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	if code := init.Run(nil); code != 0 {
+		output := done(t)
+		t.Fatalf("expected status code %d but got %d: %s", 9, code, output.All())
+	}
+
+	c := &TestCommand{
+		Meta: meta,
+	}
+
+	c.Run(nil)
+	output := done(t).All()
+
+	if !strings.Contains(output, "40 passed, 0 failed") {
+		t.Errorf("output didn't produce the right output:\n\n%s", output)
+	}
+
+	// Split the log into lines
+	lines := strings.Split(output, "\n")
+
+	// Find the positions of "test_d", "test_c", "test_setup" in the log output
+	var testDIndex, testCIndex, testSetupIndex int
+	for i, line := range lines {
+		if strings.Contains(line, "run \"setup\"") {
+			testSetupIndex = i
+		} else if strings.Contains(line, "run \"test_d\"") {
+			testDIndex = i
+		} else if strings.Contains(line, "run \"test_c\"") {
+			testCIndex = i
+		}
+	}
+	if testDIndex == 0 || testCIndex == 0 || testSetupIndex == 0 {
+		t.Fatalf("test_d, test_c, or test_setup not found in the log output")
+	}
+
+	// Ensure "test_d" appears before "test_c", because test_d has no dependencies,
+	// and would therefore run in parallel to much earlier tests which test_c depends on.
+	if testDIndex > testCIndex {
+		t.Errorf("test_d appears after test_c in the log output")
+	}
+
+	// Ensure "test_d" appears after "test_setup", because they have the same state key
+	if testDIndex < testSetupIndex {
+		t.Errorf("test_d appears before test_setup in the log output")
+	}
+}
+
 func TestTest_InterruptSkipsRemaining(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath(path.Join("test", "with_interrupt_and_additional_file")), td)
@@ -569,7 +924,7 @@ func TestTest_ProviderAlias(t *testing.T) {
 
 	output := done(t)
 
-	if code := init.Run(nil); code != 0 {
+	if code := init.Run([]string{"-no-color"}); code != 0 {
 		t.Fatalf("expected status code 0 but got %d: %s", code, output.All())
 	}
 
@@ -582,7 +937,7 @@ func TestTest_ProviderAlias(t *testing.T) {
 		Meta: meta,
 	}
 
-	code := command.Run(nil)
+	code := command.Run([]string{"-no-color"})
 	output = done(t)
 
 	printedOutput := false
@@ -601,6 +956,495 @@ func TestTest_ProviderAlias(t *testing.T) {
 	}
 }
 
+func TestTest_ComplexCondition(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "complex_condition")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+
+	providerSource, close := newMockProviderSource(t, map[string][]string{"test": {"1.0.0"}})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		Ui:               ui,
+		View:             view,
+		Streams:          streams,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	output := done(t)
+
+	if code := init.Run([]string{"-no-color"}); code != 0 {
+		t.Fatalf("expected status code 0 but got %d: %s", code, output.All())
+	}
+
+	// Reset the streams for the next command.
+	streams, done = terminal.StreamsForTesting(t)
+	meta.Streams = streams
+	meta.View = views.NewView(streams)
+
+	command := &TestCommand{
+		Meta: meta,
+	}
+
+	code := command.Run([]string{"-no-color"})
+	output = done(t)
+
+	printedOutput := false
+
+	if code != 1 {
+		printedOutput = true
+		t.Errorf("expected status code 1 but got %d: %s", code, output.All())
+	}
+
+	expectedOut := `main.tftest.hcl... in progress
+  run "validate_diff_types"... fail
+  run "validate_output"... fail
+  run "validate_complex_output"... fail
+  run "validate_complex_output_sensitive"... fail
+  run "validate_complex_output_pass"... pass
+main.tftest.hcl... tearing down
+main.tftest.hcl... fail
+
+Failure! 1 passed, 4 failed.
+`
+
+	expectedErr := `
+Error: Test assertion failed
+
+  on main.tftest.hcl line 37, in run "validate_diff_types":
+  37:     condition = var.tr1 == var.tr2 
+    ├────────────────
+    │ Warning: LHS and RHS values are of different types
+
+
+expected to fail
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 44, in run "validate_output":
+  44:     condition = output.foo == var.foo
+    ├────────────────
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │   {
+    │ -   "bar": "notbaz",
+    │ +   "bar": "baz",
+    │     "matches": "matches",
+    │ -   "qux": "quux",
+    │ -   "xuq": "xuq"
+    │ +   "qux": "qux",
+    │ +   "xuq": "nope"
+    │   }
+
+
+expected to fail due to different values
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 52, in run "validate_complex_output":
+  52:     condition = output.complex == var.bar
+    ├────────────────
+    │ Warning: LHS and RHS values are of different types
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │   {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │ -       "qux": "quux"
+    │ +       "qux": "qux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+
+
+expected to fail
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 60, in run "validate_complex_output_sensitive":
+  60:     condition = output.complex == output.complex_sensitive
+    ├────────────────
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │ - {
+    │ -   "root": [
+    │ -     {
+    │ -       "bar": [
+    │ -         1
+    │ -       ],
+    │ -       "qux": "quux"
+    │ -     },
+    │ -     {
+    │ -       "bar": [
+    │ -         2
+    │ -       ],
+    │ -       "qux": "quux"
+    │ -     }
+    │ -   ]
+    │ - }
+    │ + "(sensitive value)"
+
+
+expected to fail
+`
+	if diff := cmp.Diff(output.Stdout(), expectedOut); len(diff) > 0 {
+		t.Errorf("\nexpected: \n%s\ngot: %s\ndiff: %s", expectedOut, output.All(), diff)
+	}
+	if diff := cmp.Diff(output.Stderr(), expectedErr); len(diff) > 0 {
+		t.Errorf("\nexpected stderr: \n%s\ngot: %s\ndiff: %s", expectedErr, output.Stderr(), diff)
+	}
+
+	if provider.ResourceCount() > 0 {
+		if !printedOutput {
+			t.Errorf("should have deleted all resources on completion but left %s\n\n%s", provider.ResourceString(), output.All())
+		} else {
+			t.Errorf("should have deleted all resources on completion but left %s", provider.ResourceString())
+		}
+	}
+}
+
+func TestTest_ComplexConditionVerbose(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "complex_condition")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+
+	providerSource, close := newMockProviderSource(t, map[string][]string{"test": {"1.0.0"}})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		testingOverrides: metaOverridesForProvider(provider.Provider),
+		Ui:               ui,
+		View:             view,
+		Streams:          streams,
+		ProviderSource:   providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	output := done(t)
+
+	if code := init.Run([]string{"-no-color"}); code != 0 {
+		t.Fatalf("expected status code 0 but got %d: %s", code, output.All())
+	}
+
+	// Reset the streams for the next command.
+	streams, done = terminal.StreamsForTesting(t)
+	meta.Streams = streams
+	meta.View = views.NewView(streams)
+
+	command := &TestCommand{
+		Meta: meta,
+	}
+
+	code := command.Run([]string{"-no-color", "-verbose"})
+	output = done(t)
+
+	printedOutput := false
+
+	if code != 1 {
+		printedOutput = true
+		t.Errorf("expected status code 1 but got %d: %s", code, output.All())
+	}
+
+	expectedErr := `
+Error: Test assertion failed
+
+  on main.tftest.hcl line 37, in run "validate_diff_types":
+  37:     condition = var.tr1 == var.tr2 
+    ├────────────────
+    │ LHS:
+    │   {
+    │     "iops": null,
+    │     "size": 60
+    │   }
+    │ RHS:
+    │   {
+    │     "iops": null,
+    │     "size": 60
+    │   }
+    │ Warning: LHS and RHS values are of different types
+
+    │ var.tr1 is {
+    │     "iops": null,
+    │     "size": 60
+    │   }
+    │ var.tr2 is {
+    │     "iops": null,
+    │     "size": 60
+    │   }
+
+expected to fail
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 44, in run "validate_output":
+  44:     condition = output.foo == var.foo
+    ├────────────────
+    │ LHS:
+    │   {
+    │     "bar": "notbaz",
+    │     "matches": "matches",
+    │     "qux": "quux",
+    │     "xuq": "xuq"
+    │   }
+    │ RHS:
+    │   {
+    │     "bar": "baz",
+    │     "matches": "matches",
+    │     "qux": "qux",
+    │     "xuq": "nope"
+    │   }
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │   {
+    │ -   "bar": "notbaz",
+    │ +   "bar": "baz",
+    │     "matches": "matches",
+    │ -   "qux": "quux",
+    │ -   "xuq": "xuq"
+    │ +   "qux": "qux",
+    │ +   "xuq": "nope"
+    │   }
+
+    │ output.foo is {
+    │     "bar": "notbaz",
+    │     "matches": "matches",
+    │     "qux": "quux",
+    │     "xuq": "xuq"
+    │   }
+    │ var.foo is {
+    │     "bar": "baz",
+    │     "matches": "matches",
+    │     "qux": "qux",
+    │     "xuq": "nope"
+    │   }
+
+expected to fail due to different values
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 52, in run "validate_complex_output":
+  52:     condition = output.complex == var.bar
+    ├────────────────
+    │ LHS:
+    │   {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "quux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+    │ RHS:
+    │   {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "qux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+    │ Warning: LHS and RHS values are of different types
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │   {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │ -       "qux": "quux"
+    │ +       "qux": "qux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+
+    │ output.complex is {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "quux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+    │ var.bar is {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "qux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+
+expected to fail
+
+Error: Test assertion failed
+
+  on main.tftest.hcl line 60, in run "validate_complex_output_sensitive":
+  60:     condition = output.complex == output.complex_sensitive
+    ├────────────────
+    │ LHS:
+    │   {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "quux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+    │ RHS:
+    │   "(sensitive value)"
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │ - {
+    │ -   "root": [
+    │ -     {
+    │ -       "bar": [
+    │ -         1
+    │ -       ],
+    │ -       "qux": "quux"
+    │ -     },
+    │ -     {
+    │ -       "bar": [
+    │ -         2
+    │ -       ],
+    │ -       "qux": "quux"
+    │ -     }
+    │ -   ]
+    │ - }
+    │ + "(sensitive value)"
+
+    │ output.complex is {
+    │     "root": [
+    │       {
+    │         "bar": [
+    │           1
+    │         ],
+    │         "qux": "quux"
+    │       },
+    │       {
+    │         "bar": [
+    │           2
+    │         ],
+    │         "qux": "quux"
+    │       }
+    │     ]
+    │   }
+    │ output.complex_sensitive is "(sensitive value)"
+
+expected to fail
+`
+	outputs := []string{
+		"main.tftest.hcl... in progress",
+		"  run \"validate_diff_types\"... fail",
+		"  run \"validate_output\"... fail",
+		"  run \"validate_complex_output\"... fail",
+		"  run \"validate_complex_output_sensitive\"... fail",
+		"  run \"validate_complex_output_pass\"... pass",
+		"main.tftest.hcl... tearing down",
+		"main.tftest.hcl... fail",
+		"Failure! 1 passed, 4 failed.",
+	}
+	stdout := output.Stdout()
+	for _, expected := range outputs {
+		if !strings.Contains(stdout, expected) {
+			t.Errorf("output didn't contain expected output %q", expected)
+		}
+	}
+
+	if diff := cmp.Diff(output.Stderr(), expectedErr); len(diff) > 0 {
+		t.Errorf("\nexpected stderr: \n%s\ngot: %s\ndiff: %s", expectedErr, output.Stderr(), diff)
+	}
+
+	if provider.ResourceCount() > 0 {
+		if !printedOutput {
+			t.Errorf("should have deleted all resources on completion but left %s\n\n%s", provider.ResourceString(), output.All())
+		} else {
+			t.Errorf("should have deleted all resources on completion but left %s", provider.ResourceString())
+		}
+	}
+}
+
 func TestTest_ModuleDependencies(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath(path.Join("test", "with_setup_module")), td)
@@ -786,6 +1630,7 @@ Terraform will perform the following actions:
       + destroy_fail = (known after apply)
       + id           = "constant_value"
       + value        = "bar"
+      + write_only   = (write-only attribute)
     }
 
 Plan: 1 to add, 0 to change, 0 to destroy.
@@ -797,6 +1642,7 @@ resource "test_resource" "foo" {
     destroy_fail = false
     id           = "constant_value"
     value        = "bar"
+    write_only   = (write-only attribute)
 }
 
 main.tftest.hcl... tearing down
@@ -895,6 +1741,23 @@ it has been removed. This occurs when a provider configuration is removed
 while objects created by that provider still exist in the state. Re-add the
 provider configuration to destroy test_resource.secondary, after which you
 can remove the provider configuration again.
+`,
+		},
+		"missing-provider-definition-in-file": {
+			expectedOut: `main.tftest.hcl... in progress
+  run "passes_validation"... fail
+main.tftest.hcl... tearing down
+main.tftest.hcl... fail
+
+Failure! 0 passed, 1 failed.
+`,
+			expectedErr: `
+Error: Missing provider definition for test
+
+  on main.tftest.hcl line 12, in run "passes_validation":
+  12:     test = test
+
+This provider block references a provider definition that does not exist.
 `,
 		},
 		"missing-provider-in-test-module": {
@@ -1102,6 +1965,7 @@ resource "test_resource" "module_resource" {
     destroy_fail = false
     id           = "df6h8as9"
     value        = "start"
+    write_only   = (write-only attribute)
 }
 
   run "initial_apply"... pass
@@ -1111,6 +1975,7 @@ resource "test_resource" "resource" {
     destroy_fail = false
     id           = "598318e0"
     value        = "start"
+    write_only   = (write-only attribute)
 }
 
   run "plan_second_example"... pass
@@ -1126,6 +1991,7 @@ Terraform will perform the following actions:
       + destroy_fail = (known after apply)
       + id           = "b6a1d8cb"
       + value        = "start"
+      + write_only   = (write-only attribute)
     }
 
 Plan: 1 to add, 0 to change, 0 to destroy.
@@ -1142,7 +2008,7 @@ Terraform will perform the following actions:
   ~ resource "test_resource" "resource" {
         id           = "598318e0"
       ~ value        = "start" -> "update"
-        # (1 unchanged attribute hidden)
+        # (2 unchanged attributes hidden)
     }
 
 Plan: 0 to add, 1 to change, 0 to destroy.
@@ -1159,7 +2025,7 @@ Terraform will perform the following actions:
   ~ resource "test_resource" "module_resource" {
         id           = "df6h8as9"
       ~ value        = "start" -> "update"
-        # (1 unchanged attribute hidden)
+        # (2 unchanged attributes hidden)
     }
 
 Plan: 0 to add, 1 to change, 0 to destroy.
@@ -1172,8 +2038,8 @@ Success! 5 passed, 0 failed.
 
 	actual := output.All()
 
-	if !strings.Contains(actual, expected) {
-		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s", expected, actual)
+	if diff := cmp.Diff(expected, actual); diff != "" {
+		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expected, actual, diff)
 	}
 
 	if provider.ResourceCount() > 0 {
@@ -1418,18 +2284,6 @@ func TestTest_BadReferences(t *testing.T) {
 	}
 
 	expectedOut := `main.tftest.hcl... in progress
-  run "setup"... pass
-  run "test"... fail
-
-Warning: Value for undeclared variable
-
-  on main.tftest.hcl line 17, in run "test":
-  17:     input_three = run.madeup.response
-
-The module under test does not declare a variable named "input_three", but it
-is declared in run block "test".
-
-  run "finalise"... skip
 main.tftest.hcl... tearing down
 main.tftest.hcl... fail
 providers.tftest.hcl... in progress
@@ -1437,7 +2291,7 @@ providers.tftest.hcl... in progress
 providers.tftest.hcl... tearing down
 providers.tftest.hcl... fail
 
-Failure! 1 passed, 2 failed, 1 skipped.
+Failure! 0 passed, 1 failed.
 `
 	actualOut := output.Stdout()
 	if diff := cmp.Diff(actualOut, expectedOut); len(diff) > 0 {
@@ -1619,6 +2473,7 @@ the apply operation could not be executed and so the overall test case will
 be marked as a failure and the original diagnostic included in the test
 report.
 
+  run "no_run"... skip
 input.tftest.hcl... tearing down
 input.tftest.hcl... fail
 output.tftest.hcl... in progress
@@ -1651,7 +2506,7 @@ test report.
 resource.tftest.hcl... tearing down
 resource.tftest.hcl... fail
 
-Failure! 1 passed, 3 failed.
+Failure! 1 passed, 3 failed, 1 skipped.
 `
 	actualOut := output.Stdout()
 	if diff := cmp.Diff(expectedOut, actualOut); len(diff) > 0 {
@@ -1698,6 +2553,71 @@ input must contain the character 'b'
 	}
 }
 
+func TestTest_MissingExpectedFailuresDuringApply(t *testing.T) {
+	// Test asserting that the test run fails, but not errors out, when expected failures are not present during apply.
+	// This lets subsequent runs continue to execute and the file to be marked as failed.
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "expect_failures_during_apply")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+	view, done := testView(t)
+
+	c := &TestCommand{
+		Meta: Meta{
+			testingOverrides: metaOverridesForProvider(provider.Provider),
+			View:             view,
+		},
+	}
+
+	code := c.Run([]string{"-no-color"})
+	output := done(t)
+
+	if code == 0 {
+		t.Errorf("expected status code 0 but got %d", code)
+	}
+
+	expectedOut := `main.tftest.hcl... in progress
+  run "test"... fail
+  run "follow-up"... pass
+
+Warning: Value for undeclared variable
+
+  on main.tftest.hcl line 16, in run "follow-up":
+  16:     input = "does not matter"
+
+The module under test does not declare a variable named "input", but it is
+declared in run block "follow-up".
+
+main.tftest.hcl... tearing down
+main.tftest.hcl... fail
+
+Failure! 1 passed, 1 failed.
+`
+	actualOut := output.Stdout()
+	if diff := cmp.Diff(expectedOut, actualOut); len(diff) > 0 {
+		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expectedOut, actualOut, diff)
+	}
+
+	expectedErr := `
+Error: Missing expected failure
+
+  on main.tftest.hcl line 7, in run "test":
+   7:     output.output
+
+The checkable object, output.output, was expected to report an error but did
+not.
+`
+	actualErr := output.Stderr()
+	if diff := cmp.Diff(actualErr, expectedErr); len(diff) > 0 {
+		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expectedErr, actualErr, diff)
+	}
+
+	if provider.ResourceCount() > 0 {
+		t.Errorf("should have deleted all resources on completion but left %v", provider.ResourceString())
+	}
+}
+
 func TestTest_UnknownAndNulls(t *testing.T) {
 
 	tcs := map[string]struct {
@@ -1720,8 +2640,13 @@ Error: Test assertion failed
   on main.tftest.hcl line 8, in run "first":
    8:     condition     = test_resource.resource.value == output.null_output
     ├────────────────
-    │ output.null_output is null
-    │ test_resource.resource.value is "bar"
+    │ Warning: LHS and RHS values are of different types
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │ - "bar"
+    │ + null
+
 
 this is always going to fail
 `,
@@ -1903,8 +2828,8 @@ func TestTest_SensitiveInputValues(t *testing.T) {
 	code := c.Run([]string{"-no-color", "-verbose"})
 	output = done(t)
 
-	if code != 0 {
-		t.Errorf("expected status code 0 but got %d", code)
+	if code != 1 {
+		t.Errorf("expected status code 1 but got %d", code)
 	}
 
 	expected := `main.tftest.hcl... in progress
@@ -1923,6 +2848,22 @@ resource "test_resource" "resource" {
     destroy_fail = false
     id           = "9ddca5a9"
     value        = (sensitive value)
+    write_only   = (write-only attribute)
+}
+
+
+Outputs:
+
+password = (sensitive value)
+
+  run "test_failed"... fail
+
+# test_resource.resource:
+resource "test_resource" "resource" {
+    destroy_fail = false
+    id           = "9ddca5a9"
+    value        = (sensitive value)
+    write_only   = (write-only attribute)
 }
 
 
@@ -1931,17 +2872,58 @@ Outputs:
 password = (sensitive value)
 
 main.tftest.hcl... tearing down
-main.tftest.hcl... pass
+main.tftest.hcl... fail
 
-Success! 2 passed, 0 failed.
+Failure! 2 passed, 1 failed.
 `
 
-	actual := output.All()
+	expectedErr := `
+Error: Test assertion failed
+
+  on main.tftest.hcl line 27, in run "test_failed":
+  27:     condition = var.complex == {
+  28:       foo = "bar"
+  29:       baz = test_resource.resource.id
+  30:     }
+    ├────────────────
+    │ LHS:
+    │   {
+    │     "baz": "(sensitive value)",
+    │     "foo": "bar"
+    │   }
+    │ RHS:
+    │   {
+    │     "baz": "9ddca5a9",
+    │     "foo": "bar"
+    │   }
+    │ Diff:
+    │ --- actual
+    │ +++ expected
+    │   {
+    │ -   "baz": "(sensitive value)",
+    │ +   "baz": "9ddca5a9",
+    │     "foo": "bar"
+    │   }
+
+    │ test_resource.resource.id is "9ddca5a9"
+    │ var.complex is {
+    │     "baz": "(sensitive value)",
+    │     "foo": "bar"
+    │   }
+
+expected to fail
+`
+
+	actual := output.Stdout()
 
 	if diff := cmp.Diff(actual, expected); len(diff) > 0 {
 		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expected, actual, diff)
 	}
 
+	if diff := cmp.Diff(output.Stderr(), expectedErr); len(diff) > 0 {
+		t.Errorf("stderr didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expectedErr, output.Stderr(), diff)
+	}
+
 	if provider.ResourceCount() > 0 {
 		t.Errorf("should have deleted all resources on completion but left %v", provider.ResourceString())
 	}
@@ -2179,6 +3161,88 @@ Success! 2 passed, 0 failed.
 	}
 }
 
+func TestTest_InvalidConfig(t *testing.T) {
+	td := t.TempDir()
+	testCopyDir(t, testFixturePath(path.Join("test", "invalid_config")), td)
+	defer testChdir(t, td)()
+
+	provider := testing_command.NewProvider(nil)
+
+	providerSource, close := newMockProviderSource(t, map[string][]string{
+		"test": {"1.0.0"},
+	})
+	defer close()
+
+	streams, done := terminal.StreamsForTesting(t)
+	view := views.NewView(streams)
+	ui := new(cli.MockUi)
+
+	meta := Meta{
+		Ui:             ui,
+		View:           view,
+		Streams:        streams,
+		ProviderSource: providerSource,
+	}
+
+	init := &InitCommand{
+		Meta: meta,
+	}
+
+	output := done(t)
+
+	if code := init.Run(nil); code != 0 {
+		t.Fatalf("expected status code 0 but got %d: %s", code, output.All())
+	}
+
+	// Reset the streams for the next command.
+	streams, done = terminal.StreamsForTesting(t)
+	meta.Streams = streams
+	meta.View = views.NewView(streams)
+
+	c := &TestCommand{
+		Meta: meta,
+	}
+
+	code := c.Run([]string{"-no-color"})
+	output = done(t)
+
+	if code != 1 {
+		t.Errorf("expected status code ! but got %d", code)
+	}
+
+	expectedOut := `main.tftest.hcl... in progress
+  run "test"... fail
+main.tftest.hcl... tearing down
+main.tftest.hcl... fail
+
+Failure! 0 passed, 1 failed.
+`
+	expectedErr := `
+Error: Failed to load plugin schemas
+
+Error while loading schemas for plugin components: Failed to obtain provider
+schema: Could not load the schema for provider
+registry.terraform.io/hashicorp/test: failed to instantiate provider
+"registry.terraform.io/hashicorp/test" to obtain schema: fork/exec
+.terraform/providers/registry.terraform.io/hashicorp/test/1.0.0/%s/terraform-provider-test_1.0.0:
+permission denied..
+`
+	expectedErr = fmt.Sprintf(expectedErr, runtime.GOOS+"_"+runtime.GOARCH)
+	out := output.Stdout()
+	err := output.Stderr()
+
+	if diff := cmp.Diff(out, expectedOut); len(diff) > 0 {
+		t.Errorf("output didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expectedErr, out, diff)
+	}
+	if diff := cmp.Diff(err, expectedErr); len(diff) > 0 {
+		t.Errorf("error didn't match expected:\nexpected:\n%s\nactual:\n%s\ndiff:\n%s", expectedErr, err, diff)
+	}
+
+	if provider.ResourceCount() > 0 {
+		t.Errorf("should have deleted all resources on completion but left %v", provider.ResourceString())
+	}
+}
+
 func TestTest_RunBlocksInProviders(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath(path.Join("test", "provider_runs")), td)
@@ -2415,7 +3479,7 @@ func TestTest_JUnitOutput(t *testing.T) {
 			actualOut = timestampRegexp.ReplaceAll(actualOut, []byte("timestamp=\"TIMESTAMP_REDACTED\""))
 
 			if !bytes.Equal(actualOut, expectedOutput) {
-				t.Fatalf("wanted XML:\n%s\n got XML:\n%s\n", string(expectedOutput), string(actualOut))
+				t.Fatalf("wanted XML:\n%s\n got XML:\n%s\ndiff:%s\n", string(expectedOutput), string(actualOut), cmp.Diff(expectedOutput, actualOut))
 			}
 
 			if provider.ResourceCount() > 0 {
