@@ -12,6 +12,7 @@ import (
 	"github.com/hashicorp/go-tfe"
 	"github.com/mitchellh/colorstring"
 
+	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/command/arguments"
 	"github.com/hashicorp/terraform/internal/command/format"
 	"github.com/hashicorp/terraform/internal/command/jsonformat"
@@ -225,11 +226,10 @@ func (t *TestHuman) Run(run *moduletest.Run, file *moduletest.File, progress mod
 				}
 
 				var opts []plans.Quality
-				if !run.Verbose.Plan.CanApply() {
-					opts = append(opts, plans.NoChanges)
-				}
 				if run.Verbose.Plan.Errored {
 					opts = append(opts, plans.Errored)
+				} else if !run.Verbose.Plan.Applyable {
+					opts = append(opts, plans.NoChanges)
 				}
 
 				renderer.RenderHumanPlan(plan, run.Verbose.Plan.UIMode, opts...)
@@ -276,13 +276,15 @@ func (t *TestHuman) DestroySummary(diags tfdiags.Diagnostics, run *moduletest.Ru
 	t.Diagnostics(run, file, diags)
 
 	if state.HasManagedResourceInstanceObjects() {
+		// FIXME: This message says "resources" but this is actually a list
+		// of resource instance objects.
 		t.view.streams.Eprint(format.WordWrap(fmt.Sprintf("\nTerraform left the following resources in state after executing %s, and they need to be cleaned up manually:\n", identifier), t.view.errorColumns()))
-		for _, resource := range state.AllResourceInstanceObjectAddrs() {
+		for _, resource := range addrs.SetSortedNatural(state.AllManagedResourceInstanceObjectAddrs()) {
 			if resource.DeposedKey != states.NotDeposed {
-				t.view.streams.Eprintf("  - %s (%s)\n", resource.Instance, resource.DeposedKey)
+				t.view.streams.Eprintf("  - %s (%s)\n", resource.ResourceInstance, resource.DeposedKey)
 				continue
 			}
-			t.view.streams.Eprintf("  - %s\n", resource.Instance)
+			t.view.streams.Eprintf("  - %s\n", resource.ResourceInstance)
 		}
 	}
 }
@@ -306,12 +308,12 @@ func (t *TestHuman) FatalInterruptSummary(run *moduletest.Run, file *moduletest.
 	// with a run block.
 	if state, exists := existingStates[nil]; exists && !state.Empty() {
 		t.view.streams.Eprint(format.WordWrap("\nTerraform has already created the following resources from the module under test:\n", t.view.errorColumns()))
-		for _, resource := range state.AllResourceInstanceObjectAddrs() {
+		for _, resource := range addrs.SetSortedNatural(state.AllManagedResourceInstanceObjectAddrs()) {
 			if resource.DeposedKey != states.NotDeposed {
-				t.view.streams.Eprintf("  - %s (%s)\n", resource.Instance, resource.DeposedKey)
+				t.view.streams.Eprintf("  - %s (%s)\n", resource.ResourceInstance, resource.DeposedKey)
 				continue
 			}
-			t.view.streams.Eprintf("  - %s\n", resource.Instance)
+			t.view.streams.Eprintf("  - %s\n", resource.ResourceInstance)
 		}
 	}
 
@@ -323,12 +325,12 @@ func (t *TestHuman) FatalInterruptSummary(run *moduletest.Run, file *moduletest.
 		}
 
 		t.view.streams.Eprint(format.WordWrap(fmt.Sprintf("\nTerraform has already created the following resources for %q from %q:\n", run.Name, run.Config.Module.Source), t.view.errorColumns()))
-		for _, resource := range state.AllResourceInstanceObjectAddrs() {
+		for _, resource := range addrs.SetSortedNatural(state.AllManagedResourceInstanceObjectAddrs()) {
 			if resource.DeposedKey != states.NotDeposed {
-				t.view.streams.Eprintf("  - %s (%s)\n", resource.Instance, resource.DeposedKey)
+				t.view.streams.Eprintf("  - %s (%s)\n", resource.ResourceInstance, resource.DeposedKey)
 				continue
 			}
-			t.view.streams.Eprintf("  - %s\n", resource.Instance)
+			t.view.streams.Eprintf("  - %s\n", resource.ResourceInstance)
 		}
 	}
 
@@ -602,9 +604,9 @@ func (t *TestJSON) Run(run *moduletest.Run, file *moduletest.File, progress modu
 func (t *TestJSON) DestroySummary(diags tfdiags.Diagnostics, run *moduletest.Run, file *moduletest.File, state *states.State) {
 	if state.HasManagedResourceInstanceObjects() {
 		cleanup := json.TestFileCleanup{}
-		for _, resource := range state.AllResourceInstanceObjectAddrs() {
+		for _, resource := range addrs.SetSortedNatural(state.AllManagedResourceInstanceObjectAddrs()) {
 			cleanup.FailedResources = append(cleanup.FailedResources, json.TestFailedResource{
-				Instance:   resource.Instance.String(),
+				Instance:   resource.ResourceInstance.String(),
 				DeposedKey: resource.DeposedKey.String(),
 			})
 		}
@@ -660,9 +662,9 @@ func (t *TestJSON) FatalInterruptSummary(run *moduletest.Run, file *moduletest.F
 		}
 
 		var resources []json.TestFailedResource
-		for _, resource := range state.AllResourceInstanceObjectAddrs() {
+		for _, resource := range addrs.SetSortedNatural(state.AllManagedResourceInstanceObjectAddrs()) {
 			resources = append(resources, json.TestFailedResource{
-				Instance:   resource.Instance.String(),
+				Instance:   resource.ResourceInstance.String(),
 				DeposedKey: resource.DeposedKey.String(),
 			})
 		}
