@@ -47,7 +47,7 @@ func TestModuleInstaller(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, nil)
 	_, diags := inst.InstallModules(context.Background(), ".", "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	wantCalls := []testInstallHookCall{
 		{
@@ -78,7 +78,7 @@ func TestModuleInstaller(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfig(".")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	wantTraces := map[string]string{
 		"":                "in root module",
@@ -380,7 +380,7 @@ func TestModuleInstaller_symlink(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, nil)
 	_, diags := inst.InstallModules(context.Background(), ".", "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	wantCalls := []testInstallHookCall{
 		{
@@ -411,7 +411,7 @@ func TestModuleInstaller_symlink(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfig(".")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	wantTraces := map[string]string{
 		"":                "in root module",
@@ -460,7 +460,7 @@ func TestLoaderInstallModules_invalidRegistry(t *testing.T) {
 	if !diags.HasErrors() {
 		t.Fatal("expected error")
 	} else {
-		assertDiagnosticCount(t, diags, 1)
+		tfdiags.AssertDiagnosticCount(t, diags, 1)
 		assertDiagnosticSummary(t, diags, "Unreadable module subdirectory")
 
 		// the diagnostic should specifically call out the submodule that failed
@@ -495,7 +495,7 @@ func TestLoaderInstallModules_registry(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, registry.NewClient(nil, nil))
 	_, diags := inst.InstallModules(context.Background(), dir, "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	v := version.Must(version.NewVersion("0.0.1"))
 
@@ -609,7 +609,7 @@ func TestLoaderInstallModules_registry(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfig(".")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	wantTraces := map[string]string{
 		"":                             "in local caller for registry-modules",
@@ -658,7 +658,7 @@ func TestLoaderInstallModules_goGetter(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, registry.NewClient(nil, nil))
 	_, diags := inst.InstallModules(context.Background(), dir, "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	wantCalls := []testInstallHookCall{
 		// the configuration builder visits each level of calls in lexicographical
@@ -739,7 +739,7 @@ func TestLoaderInstallModules_goGetter(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfig(".")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	wantTraces := map[string]string{
 		"":                             "in local caller for go-getter-modules",
@@ -776,7 +776,7 @@ func TestModuleInstaller_fromTests(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, nil)
 	_, diags := inst.InstallModules(context.Background(), ".", "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	wantCalls := []testInstallHookCall{
 		{
@@ -801,7 +801,7 @@ func TestModuleInstaller_fromTests(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfigWithTests(".", "tests")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	if config.Module.Tests["tests/main.tftest.hcl"].Runs[0].ConfigUnderTest == nil {
 		t.Fatalf("should have loaded config into the relevant run block but did not")
@@ -833,7 +833,7 @@ func TestLoadInstallModules_registryFromTest(t *testing.T) {
 	defer close()
 	inst := NewModuleInstaller(modulesDir, loader, registry.NewClient(nil, nil))
 	_, diags := inst.InstallModules(context.Background(), dir, "tests", false, false, hooks)
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	v := version.Must(version.NewVersion("0.0.1"))
 	wantCalls := []testInstallHookCall{
@@ -910,7 +910,7 @@ func TestLoadInstallModules_registryFromTest(t *testing.T) {
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
 	config, loadDiags := loader.LoadConfigWithTests(".", "tests")
-	assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
+	tfdiags.AssertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags))
 
 	if config.Module.Tests["main.tftest.hcl"].Runs[0].ConfigUnderTest == nil {
 		t.Fatalf("should have loaded config into the relevant run block but did not")
@@ -997,23 +997,6 @@ func tempChdir(t *testing.T, sourceDir string) (string, func()) {
 	}
 }
 
-func assertNoDiagnostics(t *testing.T, diags tfdiags.Diagnostics) bool {
-	t.Helper()
-	return assertDiagnosticCount(t, diags, 0)
-}
-
-func assertDiagnosticCount(t *testing.T, diags tfdiags.Diagnostics, want int) bool {
-	t.Helper()
-	if len(diags) != want {
-		t.Errorf("wrong number of diagnostics %d; want %d", len(diags), want)
-		for _, diag := range diags {
-			t.Logf("- %#v", diag)
-		}
-		return true
-	}
-	return false
-}
-
 func assertDiagnosticSummary(t *testing.T, diags tfdiags.Diagnostics, want string) bool {
 	t.Helper()
 
