@@ -9,11 +9,158 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/zclconf/go-cty-debug/ctydebug"
 	"github.com/zclconf/go-cty/cty"
 
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/configs"
+	"github.com/hashicorp/terraform/internal/moduletest/mocking"
 )
 
+func TestExpanderWithOverrides(t *testing.T) {
+
+	mustModuleInstance := func(t *testing.T, s string) addrs.ModuleInstance {
+		if len(s) == 0 {
+			return addrs.RootModuleInstance
+		}
+
+		addr, diags := addrs.ParseModuleInstanceStr(s)
+		if diags.HasErrors() {
+			t.Fatalf("unexpected error: %s", diags.Err())
+		}
+		return addr
+	}
+
+	tcs := map[string]struct {
+		// Hook to install chosen overrides.
+		overrides mocking.InitLocalOverrides
+
+		// Hook to initialise the expander with the desired state.
+		expander func(*Expander)
+
+		// The target module instance to inspect.
+		target string
+
+		// Set to true to include overrides in the result.
+		includeOverrides bool
+
+		// The expected result.
+		wantModules []addrs.ModuleInstance
+
+		// The expected result for partial modules.
+		wantPartials map[string]bool
+	}{
+		"root module": {
+			wantModules:  singletonRootModule,
+			wantPartials: make(map[string]bool),
+		},
+		"instanced child module not overridden": {
+			expander: func(expander *Expander) {
+				expander.SetModuleCount(addrs.RootModuleInstance, addrs.ModuleCall{Name: "double"}, 2)
+			},
+			target: "module.double",
+			wantModules: []addrs.ModuleInstance{
+				mustModuleInstance(t, "module.double[0]"),
+				mustModuleInstance(t, "module.double[1]"),
+			},
+			wantPartials: make(map[string]bool),
+		},
+		"instanced child module single instance overridden": {
+			overrides: func(overrides addrs.Map[addrs.Targetable, *configs.Override]) {
+				overrides.Put(mustModuleInstance(t, "module.double[0]"), &configs.Override{})
+			},
+			expander: func(expander *Expander) {
+				expander.SetModuleCount(addrs.RootModuleInstance, addrs.ModuleCall{Name: "double"}, 2)
+			},
+			target: "module.double",
+			wantModules: []addrs.ModuleInstance{
+				mustModuleInstance(t, "module.double[1]"),
+			},
+			wantPartials: make(map[string]bool),
+		},
+		"instanced child module single instance overridden includes overrides": {
+			overrides: func(overrides addrs.Map[addrs.Targetable, *configs.Override]) {
+				overrides.Put(mustModuleInstance(t, "module.double[0]"), &configs.Override{})
+			},
+			expander: func(expander *Expander) {
+				expander.SetModuleCount(addrs.RootModuleInstance, addrs.ModuleCall{Name: "double"}, 2)
+			},
+			target:           "module.double",
+			includeOverrides: true,
+			wantModules: []addrs.ModuleInstance{
+				mustModuleInstance(t, "module.double[0]"),
+				mustModuleInstance(t, "module.double[1]"),
+			},
+			wantPartials: make(map[string]bool),
+		},
+		"deeply nested child module with parent overridden": {
+			overrides: func(overrides addrs.Map[addrs.Targetable, *configs.Override]) {
+				overrides.Put(mustModuleInstance(t, "module.double[0]"), &configs.Override{})
+			},
+			expander: func(expander *Expander) {
+				expander.SetModuleCount(addrs.RootModuleInstance, addrs.ModuleCall{Name: "double"}, 2)
+				expander.SetModuleSingle(mustModuleInstance(t, "module.double[1]"), addrs.ModuleCall{Name: "single"})
+			},
+			target:       "module.double.module.single",
+			wantModules:  []addrs.ModuleInstance{mustModuleInstance(t, "module.double[1].module.single")},
+			wantPartials: make(map[string]bool),
+		},
+		"unknown child module overridden by instanced module": {
+			overrides: func(overrides addrs.Map[addrs.Targetable, *configs.Override]) {
+				overrides.Put(mustModuleInstance(t, "module.unknown[0]"), &configs.Override{})
+			},
+			expander: func(expander *Expander) {
+				expander.SetModuleCountUnknown(addrs.RootModuleInstance, addrs.ModuleCall{Name: "unknown"})
+			},
+			target: "module.unknown",
+			wantPartials: map[string]bool{
+				"module.unknown[*]": true,
+			},
+		},
+		"unknown child module overridden by instanced module includes overrides": {
+			overrides: func(overrides addrs.Map[addrs.Targetable, *configs.Override]) {
+				overrides.Put(mustModuleInstance(t, "module.unknown"), &configs.Override{})
+			},
+			expander: func(expander *Expander) {
+				expander.SetModuleCountUnknown(addrs.RootModuleInstance, addrs.ModuleCall{Name: "unknown"})
+			},
+			target:       "module.unknown",
+			wantPartials: make(map[string]bool), // This time it's empty, as we overrode all instances.
+		},
+	}
+
+	for name, tc := range tcs {
+		t.Run(name, func(t *testing.T) {
+			overrides := mocking.OverridesForTesting(nil, tc.overrides)
+
+			expander := NewExpander(overrides)
+			if tc.expander != nil {
+				tc.expander(expander)
+			}
+
+			target := mustModuleInstance(t, tc.target).Module()
+
+			gotModules := expander.ExpandModule(target, tc.includeOverrides)
+			gotPartials := expander.UnknownModuleInstances(target, tc.includeOverrides)
+
+			if diff := cmp.Diff(tc.wantModules, gotModules); len(diff) > 0 {
+				t.Errorf("wrong result\n%s", diff)
+			}
+
+			// Convert the gotPartials into strings to make cmp.Diff work.
+			gotPartialsStr := make(map[string]bool, len(gotPartials))
+			for _, partial := range gotPartials {
+				gotPartialsStr[partial.String()] = true
+			}
+
+			if diff := cmp.Diff(tc.wantPartials, gotPartialsStr, ctydebug.CmpOptions); len(diff) > 0 {
+				t.Errorf("wrong result\n%s", diff)
+			}
+		})
+	}
+
+}
+
 func TestExpander(t *testing.T) {
 	// Some module and resource addresses and values we'll use repeatedly below.
 	singleModuleAddr := addrs.ModuleCall{Name: "single"}
@@ -70,7 +217,7 @@ func TestExpander(t *testing.T) {
 	//     - resource test.single with no count or for_each
 	//     - resource test.count2 with count = 2
 
-	ex := NewExpander()
+	ex := NewExpander(nil)
 
 	// We don't register the root module, because it's always implied to exist.
 	//
@@ -126,7 +273,7 @@ func TestExpander(t *testing.T) {
 	t.Run("root module", func(t *testing.T) {
 		// Requesting expansion of the root module doesn't really mean anything
 		// since it's always a singleton, but for consistency it should work.
-		got := ex.ExpandModule(addrs.RootModule)
+		got := ex.ExpandModule(addrs.RootModule, false)
 		want := []addrs.ModuleInstance{addrs.RootModuleInstance}
 		if diff := cmp.Diff(want, got); diff != "" {
 			t.Errorf("wrong result\n%s", diff)
@@ -181,7 +328,7 @@ func TestExpander(t *testing.T) {
 		}
 	})
 	t.Run("module single", func(t *testing.T) {
-		got := ex.ExpandModule(addrs.RootModule.Child("single"))
+		got := ex.ExpandModule(addrs.RootModule.Child("single"), false)
 		want := []addrs.ModuleInstance{
 			mustModuleInstanceAddr(`module.single`),
 		}
@@ -248,7 +395,7 @@ func TestExpander(t *testing.T) {
 		}
 	})
 	t.Run("module count2", func(t *testing.T) {
-		got := ex.ExpandModule(mustModuleAddr(`count2`))
+		got := ex.ExpandModule(mustModuleAddr(`count2`), false)
 		want := []addrs.ModuleInstance{
 			mustModuleInstanceAddr(`module.count2[0]`),
 			mustModuleInstanceAddr(`module.count2[1]`),
@@ -286,7 +433,7 @@ func TestExpander(t *testing.T) {
 		}
 	})
 	t.Run("module count2 module count2", func(t *testing.T) {
-		got := ex.ExpandModule(mustModuleAddr(`count2.count2`))
+		got := ex.ExpandModule(mustModuleAddr(`count2.count2`), false)
 		want := []addrs.ModuleInstance{
 			mustModuleInstanceAddr(`module.count2[0].module.count2[0]`),
 			mustModuleInstanceAddr(`module.count2[0].module.count2[1]`),
@@ -297,6 +444,24 @@ func TestExpander(t *testing.T) {
 			t.Errorf("wrong result\n%s", diff)
 		}
 	})
+	t.Run("module count2[0] module count2 instances", func(t *testing.T) {
+		instAddr := mustModuleInstanceAddr(`module.count2[0].module.count2[0]`)
+		callAddr := instAddr.AbsCall() // discards the final [0] instance key from the above
+		keyType, got, known := ex.ExpandAbsModuleCall(callAddr)
+		if !known {
+			t.Fatal("expansion unknown; want known")
+		}
+		if keyType != addrs.IntKeyType {
+			t.Fatalf("wrong key type %#v; want %#v", keyType, addrs.IntKeyType)
+		}
+		want := []addrs.InstanceKey{
+			addrs.IntKey(0),
+			addrs.IntKey(1),
+		}
+		if diff := cmp.Diff(want, got); diff != "" {
+			t.Errorf("wrong result\n%s", diff)
+		}
+	})
 	t.Run("module count2 module count2 GetDeepestExistingModuleInstance", func(t *testing.T) {
 		t.Run("first step invalid", func(t *testing.T) {
 			got := ex.GetDeepestExistingModuleInstance(mustModuleInstanceAddr(`module.count2["nope"].module.count2[0]`))
@@ -359,7 +524,7 @@ func TestExpander(t *testing.T) {
 		}
 	})
 	t.Run("module count0", func(t *testing.T) {
-		got := ex.ExpandModule(mustModuleAddr(`count0`))
+		got := ex.ExpandModule(mustModuleAddr(`count0`), false)
 		want := []addrs.ModuleInstance(nil)
 		if diff := cmp.Diff(want, got); diff != "" {
 			t.Errorf("wrong result\n%s", diff)
@@ -379,7 +544,7 @@ func TestExpander(t *testing.T) {
 		}
 	})
 	t.Run("module for_each", func(t *testing.T) {
-		got := ex.ExpandModule(mustModuleAddr(`for_each`))
+		got := ex.ExpandModule(mustModuleAddr(`for_each`), false)
 		want := []addrs.ModuleInstance{
 			mustModuleInstanceAddr(`module.for_each["a"]`),
 			mustModuleInstanceAddr(`module.for_each["b"]`),
@@ -499,6 +664,163 @@ func TestExpander(t *testing.T) {
 	})
 }
 
+func TestExpanderWithUnknowns(t *testing.T) {
+	t.Run("resource in root module with unknown for_each", func(t *testing.T) {
+		resourceAddr := addrs.Resource{
+			Mode: addrs.ManagedResourceMode,
+			Type: "test",
+			Name: "foo",
+		}
+		ex := NewExpander(nil)
+		ex.SetResourceForEachUnknown(addrs.RootModuleInstance, resourceAddr)
+
+		got := ex.ExpandModuleResource(addrs.RootModule, resourceAddr)
+		if len(got) != 0 {
+			t.Errorf("unexpected known addresses: %#v", got)
+		}
+	})
+	t.Run("resource in root module with unknown count", func(t *testing.T) {
+		resourceAddr := addrs.Resource{
+			Mode: addrs.ManagedResourceMode,
+			Type: "test",
+			Name: "foo",
+		}
+		ex := NewExpander(nil)
+		ex.SetResourceCountUnknown(addrs.RootModuleInstance, resourceAddr)
+
+		got := ex.ExpandModuleResource(addrs.RootModule, resourceAddr)
+		if len(got) != 0 {
+			t.Errorf("unexpected known addresses: %#v", got)
+		}
+	})
+	t.Run("module with unknown for_each", func(t *testing.T) {
+		moduleCallAddr := addrs.ModuleCall{Name: "foo"}
+		ex := NewExpander(nil)
+		ex.SetModuleForEachUnknown(addrs.RootModuleInstance, moduleCallAddr)
+
+		got := ex.ExpandModule(addrs.Module{moduleCallAddr.Name}, false)
+		if len(got) != 0 {
+			t.Errorf("unexpected known addresses: %#v", got)
+		}
+
+		gotUnknown := ex.UnknownModuleInstances(addrs.Module{moduleCallAddr.Name}, false)
+		if len(gotUnknown) != 1 {
+			t.Errorf("unexpected unknown addresses: %#v", gotUnknown)
+		}
+		wantUnknownCall := addrs.RootModuleInstance.UnexpandedChild(moduleCallAddr)
+		if !gotUnknown.Has(wantUnknownCall) {
+			t.Errorf("unknown should have %s, but it doesn't", wantUnknownCall)
+		}
+	})
+	t.Run("module with unknown count", func(t *testing.T) {
+		moduleCallAddr := addrs.ModuleCall{Name: "foo"}
+		ex := NewExpander(nil)
+		ex.SetModuleCountUnknown(addrs.RootModuleInstance, moduleCallAddr)
+
+		gotKnown := ex.ExpandModule(addrs.Module{moduleCallAddr.Name}, false)
+		if len(gotKnown) != 0 {
+			t.Errorf("unexpected known addresses: %#v", gotKnown)
+		}
+
+		gotUnknown := ex.UnknownModuleInstances(addrs.Module{moduleCallAddr.Name}, false)
+		if len(gotUnknown) != 1 {
+			t.Errorf("unexpected unknown addresses: %#v", gotUnknown)
+		}
+		wantUnknownCall := addrs.RootModuleInstance.UnexpandedChild(moduleCallAddr)
+		if !gotUnknown.Has(wantUnknownCall) {
+			t.Errorf("unknown should have %s, but it doesn't", wantUnknownCall)
+		}
+	})
+	t.Run("nested module with unknown count", func(t *testing.T) {
+		moduleCallAddr1 := addrs.ModuleCall{Name: "foo"}
+		moduleCallAddr2 := addrs.ModuleCall{Name: "bar"}
+		module1 := addrs.RootModule.Child(moduleCallAddr1.Name)
+		module2 := module1.Child(moduleCallAddr2.Name)
+		module1Inst0 := addrs.RootModuleInstance.Child("foo", addrs.IntKey(0))
+		module1Inst1 := addrs.RootModuleInstance.Child("foo", addrs.IntKey(1))
+		module1Inst2 := addrs.RootModuleInstance.Child("foo", addrs.IntKey(2))
+		ex := NewExpander(nil)
+		ex.SetModuleCount(addrs.RootModuleInstance, moduleCallAddr1, 3)
+		ex.SetModuleCountUnknown(module1Inst0, moduleCallAddr2)
+		ex.SetModuleCount(module1Inst1, moduleCallAddr2, 1)
+		ex.SetModuleCountUnknown(module1Inst2, moduleCallAddr2)
+
+		// We'll also put some resources inside module.foo[1].module.bar[0]
+		// so that we can test requesting unknown resource instance sets.
+		resourceAddrKnownExp := addrs.Resource{
+			Mode: addrs.ManagedResourceMode,
+			Type: "test",
+			Name: "known_expansion",
+		}
+		resourceAddrUnknownExp := addrs.Resource{
+			Mode: addrs.ManagedResourceMode,
+			Type: "test",
+			Name: "unknown_expansion",
+		}
+		module1Inst1Module2Inst0 := module1Inst1.Child("bar", addrs.IntKey(0))
+		ex.SetResourceCount(module1Inst1Module2Inst0, resourceAddrKnownExp, 2)
+		ex.SetResourceCountUnknown(module1Inst1Module2Inst0, resourceAddrUnknownExp)
+
+		module2Call := addrs.AbsModuleCall{
+			Module: module1Inst0,
+			Call:   moduleCallAddr2,
+		}
+		_, _, instsKnown := ex.ExpandAbsModuleCall(module2Call)
+		if instsKnown {
+			t.Fatalf("instances of %s are known; should be unknown", module2Call.String())
+		}
+
+		gotKnown := ex.ExpandModule(module2, false)
+		wantKnown := []addrs.ModuleInstance{
+			module1Inst1.Child("bar", addrs.IntKey(0)),
+		}
+		if diff := cmp.Diff(wantKnown, gotKnown); diff != "" {
+			t.Errorf("unexpected known addresses\n%s", diff)
+		}
+
+		gotUnknown := ex.UnknownModuleInstances(module2, false)
+		if len(gotUnknown) != 2 {
+			t.Errorf("unexpected unknown addresses: %#v", gotUnknown)
+		}
+		if wantUnknownCall := module1Inst0.UnexpandedChild(moduleCallAddr2); !gotUnknown.Has(wantUnknownCall) {
+			t.Errorf("unknown should have %s, but it doesn't", wantUnknownCall)
+		}
+		if unwantUnknownCall := module1Inst1.UnexpandedChild(moduleCallAddr2); gotUnknown.Has(unwantUnknownCall) {
+			t.Errorf("unknown should not have %s, but does", unwantUnknownCall)
+		}
+		if wantUnknownCall := module1Inst2.UnexpandedChild(moduleCallAddr2); !gotUnknown.Has(wantUnknownCall) {
+			t.Errorf("unknown should have %s, but it doesn't", wantUnknownCall)
+		}
+
+		gotKnownResource := ex.ExpandResource(module1Inst1Module2Inst0.Resource(
+			resourceAddrKnownExp.Mode, resourceAddrKnownExp.Type, resourceAddrKnownExp.Name,
+		))
+		wantKnownResource := []addrs.AbsResourceInstance{
+			mustAbsResourceInstanceAddr("module.foo[1].module.bar[0].test.known_expansion[0]"),
+			mustAbsResourceInstanceAddr("module.foo[1].module.bar[0].test.known_expansion[1]"),
+		}
+		if diff := cmp.Diff(wantKnownResource, gotKnownResource); diff != "" {
+			t.Errorf("unexpected known addresses\n%s", diff)
+		}
+
+		gotUnknownResource := ex.UnknownResourceInstances(module2.Resource(
+			resourceAddrUnknownExp.Mode, resourceAddrUnknownExp.Type, resourceAddrUnknownExp.Name,
+		))
+		if len(gotUnknownResource) != 3 {
+			t.Errorf("unexpected unknown addresses: %#v", gotUnknownResource)
+		}
+		if wantResInst := module1Inst0.UnexpandedChild(moduleCallAddr2).Resource(resourceAddrUnknownExp); !gotUnknownResource.Has(wantResInst) {
+			t.Errorf("unknown should have %s, but it doesn't", wantResInst)
+		}
+		if wantResInst := module1Inst1Module2Inst0.UnexpandedResource(resourceAddrUnknownExp); !gotUnknownResource.Has(wantResInst) {
+			t.Errorf("unknown should have %s, but it doesn't", wantResInst)
+		}
+		if wantResInst := module1Inst2.UnexpandedChild(moduleCallAddr2).Resource(resourceAddrUnknownExp); !gotUnknownResource.Has(wantResInst) {
+			t.Errorf("unknown should have %s, but it doesn't", wantResInst)
+		}
+	})
+}
+
 func mustAbsResourceInstanceAddr(str string) addrs.AbsResourceInstance {
 	addr, diags := addrs.ParseAbsResourceInstanceStr(str)
 	if diags.HasErrors() {
