@@ -4,8 +4,6 @@
 package terraform
 
 import (
-	"bufio"
-	"bytes"
 	"fmt"
 	"log"
 	"reflect"
@@ -15,7 +13,6 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
 	"github.com/zclconf/go-cty/cty"
@@ -44,356 +41,6 @@ const (
 // multiVal matches the index key to a flatmapped set, list or map
 var multiVal = regexp.MustCompile(`\.(#|%)$`)
 
-// Diff tracks the changes that are necessary to apply a configuration
-// to an existing infrastructure.
-type Diff struct {
-	// Modules contains all the modules that have a diff
-	Modules []*ModuleDiff
-}
-
-// Prune cleans out unused structures in the diff without affecting
-// the behavior of the diff at all.
-//
-// This is not safe to call concurrently. This is safe to call on a
-// nil Diff.
-func (d *Diff) Prune() {
-	if d == nil {
-		return
-	}
-
-	// Prune all empty modules
-	newModules := make([]*ModuleDiff, 0, len(d.Modules))
-	for _, m := range d.Modules {
-		// If the module isn't empty, we keep it
-		if !m.Empty() {
-			newModules = append(newModules, m)
-		}
-	}
-	if len(newModules) == 0 {
-		newModules = nil
-	}
-	d.Modules = newModules
-}
-
-// AddModule adds the module with the given path to the diff.
-//
-// This should be the preferred method to add module diffs since it
-// allows us to optimize lookups later as well as control sorting.
-func (d *Diff) AddModule(path addrs.ModuleInstance) *ModuleDiff {
-	// Lower the new-style address into a legacy-style address.
-	// This requires that none of the steps have instance keys, which is
-	// true for all addresses at the time of implementing this because
-	// "count" and "for_each" are not yet implemented for modules.
-	legacyPath := make([]string, len(path))
-	for i, step := range path {
-		if step.InstanceKey != addrs.NoKey {
-			// FIXME: Once the rest of Terraform is ready to use count and
-			// for_each, remove all of this and just write the addrs.ModuleInstance
-			// value itself into the ModuleState.
-			panic("diff cannot represent modules with count or for_each keys")
-		}
-
-		legacyPath[i] = step.Name
-	}
-
-	m := &ModuleDiff{Path: legacyPath}
-	m.init()
-	d.Modules = append(d.Modules, m)
-	return m
-}
-
-// ModuleByPath is used to lookup the module diff for the given path.
-// This should be the preferred lookup mechanism as it allows for future
-// lookup optimizations.
-func (d *Diff) ModuleByPath(path addrs.ModuleInstance) *ModuleDiff {
-	if d == nil {
-		return nil
-	}
-	for _, mod := range d.Modules {
-		if mod.Path == nil {
-			panic("missing module path")
-		}
-		modPath := normalizeModulePath(mod.Path)
-		if modPath.String() == path.String() {
-			return mod
-		}
-	}
-	return nil
-}
-
-// RootModule returns the ModuleState for the root module
-func (d *Diff) RootModule() *ModuleDiff {
-	root := d.ModuleByPath(addrs.RootModuleInstance)
-	if root == nil {
-		panic("missing root module")
-	}
-	return root
-}
-
-// Empty returns true if the diff has no changes.
-func (d *Diff) Empty() bool {
-	if d == nil {
-		return true
-	}
-
-	for _, m := range d.Modules {
-		if !m.Empty() {
-			return false
-		}
-	}
-
-	return true
-}
-
-// Equal compares two diffs for exact equality.
-//
-// This is different from the Same comparison that is supported which
-// checks for operation equality taking into account computed values. Equal
-// instead checks for exact equality.
-func (d *Diff) Equal(d2 *Diff) bool {
-	// If one is nil, they must both be nil
-	if d == nil || d2 == nil {
-		return d == d2
-	}
-
-	// Sort the modules
-	sort.Sort(moduleDiffSort(d.Modules))
-	sort.Sort(moduleDiffSort(d2.Modules))
-
-	// Copy since we have to modify the module destroy flag to false so
-	// we don't compare that. TODO: delete this when we get rid of the
-	// destroy flag on modules.
-	dCopy := d.DeepCopy()
-	d2Copy := d2.DeepCopy()
-	for _, m := range dCopy.Modules {
-		m.Destroy = false
-	}
-	for _, m := range d2Copy.Modules {
-		m.Destroy = false
-	}
-
-	// Use DeepEqual
-	return reflect.DeepEqual(dCopy, d2Copy)
-}
-
-// DeepCopy performs a deep copy of all parts of the Diff, making the
-// resulting Diff safe to use without modifying this one.
-func (d *Diff) DeepCopy() *Diff {
-	copy, err := copystructure.Config{Lock: true}.Copy(d)
-	if err != nil {
-		panic(err)
-	}
-
-	return copy.(*Diff)
-}
-
-func (d *Diff) String() string {
-	var buf bytes.Buffer
-
-	keys := make([]string, 0, len(d.Modules))
-	lookup := make(map[string]*ModuleDiff)
-	for _, m := range d.Modules {
-		addr := normalizeModulePath(m.Path)
-		key := addr.String()
-		keys = append(keys, key)
-		lookup[key] = m
-	}
-	sort.Strings(keys)
-
-	for _, key := range keys {
-		m := lookup[key]
-		mStr := m.String()
-
-		// If we're the root module, we just write the output directly.
-		if reflect.DeepEqual(m.Path, rootModulePath) {
-			buf.WriteString(mStr + "\n")
-			continue
-		}
-
-		buf.WriteString(fmt.Sprintf("%s:\n", key))
-
-		s := bufio.NewScanner(strings.NewReader(mStr))
-		for s.Scan() {
-			buf.WriteString(fmt.Sprintf("  %s\n", s.Text()))
-		}
-	}
-
-	return strings.TrimSpace(buf.String())
-}
-
-func (d *Diff) init() {
-	if d.Modules == nil {
-		rootDiff := &ModuleDiff{Path: rootModulePath}
-		d.Modules = []*ModuleDiff{rootDiff}
-	}
-	for _, m := range d.Modules {
-		m.init()
-	}
-}
-
-// ModuleDiff tracks the differences between resources to apply within
-// a single module.
-type ModuleDiff struct {
-	Path      []string
-	Resources map[string]*InstanceDiff
-	Destroy   bool // Set only by the destroy plan
-}
-
-func (d *ModuleDiff) init() {
-	if d.Resources == nil {
-		d.Resources = make(map[string]*InstanceDiff)
-	}
-	for _, r := range d.Resources {
-		r.init()
-	}
-}
-
-// ChangeType returns the type of changes that the diff for this
-// module includes.
-//
-// At a module level, this will only be DiffNone, DiffUpdate, DiffDestroy, or
-// DiffCreate. If an instance within the module has a DiffDestroyCreate
-// then this will register as a DiffCreate for a module.
-func (d *ModuleDiff) ChangeType() DiffChangeType {
-	result := DiffNone
-	for _, r := range d.Resources {
-		change := r.ChangeType()
-		switch change {
-		case DiffCreate, DiffDestroy:
-			if result == DiffNone {
-				result = change
-			}
-		case DiffDestroyCreate, DiffUpdate:
-			result = DiffUpdate
-		}
-	}
-
-	return result
-}
-
-// Empty returns true if the diff has no changes within this module.
-func (d *ModuleDiff) Empty() bool {
-	if d.Destroy {
-		return false
-	}
-
-	if len(d.Resources) == 0 {
-		return true
-	}
-
-	for _, rd := range d.Resources {
-		if !rd.Empty() {
-			return false
-		}
-	}
-
-	return true
-}
-
-// Instances returns the instance diffs for the id given. This can return
-// multiple instance diffs if there are counts within the resource.
-func (d *ModuleDiff) Instances(id string) []*InstanceDiff {
-	var result []*InstanceDiff
-	for k, diff := range d.Resources {
-		if k == id || strings.HasPrefix(k, id+".") {
-			if !diff.Empty() {
-				result = append(result, diff)
-			}
-		}
-	}
-
-	return result
-}
-
-// IsRoot says whether or not this module diff is for the root module.
-func (d *ModuleDiff) IsRoot() bool {
-	return reflect.DeepEqual(d.Path, rootModulePath)
-}
-
-// String outputs the diff in a long but command-line friendly output
-// format that users can read to quickly inspect a diff.
-func (d *ModuleDiff) String() string {
-	var buf bytes.Buffer
-
-	names := make([]string, 0, len(d.Resources))
-	for name, _ := range d.Resources {
-		names = append(names, name)
-	}
-	sort.Strings(names)
-
-	for _, name := range names {
-		rdiff := d.Resources[name]
-
-		crud := "UPDATE"
-		switch {
-		case rdiff.RequiresNew() && (rdiff.GetDestroy() || rdiff.GetDestroyTainted()):
-			crud = "DESTROY/CREATE"
-		case rdiff.GetDestroy() || rdiff.GetDestroyDeposed():
-			crud = "DESTROY"
-		case rdiff.RequiresNew():
-			crud = "CREATE"
-		}
-
-		extra := ""
-		if !rdiff.GetDestroy() && rdiff.GetDestroyDeposed() {
-			extra = " (deposed only)"
-		}
-
-		buf.WriteString(fmt.Sprintf(
-			"%s: %s%s\n",
-			crud,
-			name,
-			extra))
-
-		keyLen := 0
-		rdiffAttrs := rdiff.CopyAttributes()
-		keys := make([]string, 0, len(rdiffAttrs))
-		for key, _ := range rdiffAttrs {
-			if key == "id" {
-				continue
-			}
-
-			keys = append(keys, key)
-			if len(key) > keyLen {
-				keyLen = len(key)
-			}
-		}
-		sort.Strings(keys)
-
-		for _, attrK := range keys {
-			attrDiff, _ := rdiff.GetAttribute(attrK)
-
-			v := attrDiff.New
-			u := attrDiff.Old
-			if attrDiff.NewComputed {
-				v = "<computed>"
-			}
-
-			if attrDiff.Sensitive {
-				u = "<sensitive>"
-				v = "<sensitive>"
-			}
-
-			updateMsg := ""
-			if attrDiff.RequiresNew {
-				updateMsg = " (forces new resource)"
-			} else if attrDiff.Sensitive {
-				updateMsg = " (attribute changed)"
-			}
-
-			buf.WriteString(fmt.Sprintf(
-				"  %s:%s %#v => %#v%s\n",
-				attrK,
-				strings.Repeat(" ", keyLen-len(attrK)),
-				u,
-				v,
-				updateMsg))
-		}
-	}
-
-	return buf.String()
-}
-
 // InstanceDiff is the diff of a resource from some state to another.
 type InstanceDiff struct {
 	mu             sync.Mutex
@@ -1434,21 +1081,3 @@ func (d *InstanceDiff) Same(d2 *InstanceDiff) (bool, string) {
 
 	return true, ""
 }
-
-// moduleDiffSort implements sort.Interface to sort module diffs by path.
-type moduleDiffSort []*ModuleDiff
-
-func (s moduleDiffSort) Len() int      { return len(s) }
-func (s moduleDiffSort) Swap(i, j int) { s[i], s[j] = s[j], s[i] }
-func (s moduleDiffSort) Less(i, j int) bool {
-	a := s[i]
-	b := s[j]
-
-	// If the lengths are different, then the shorter one always wins
-	if len(a.Path) != len(b.Path) {
-		return len(a.Path) < len(b.Path)
-	}
-
-	// Otherwise, compare lexically
-	return strings.Join(a.Path, ".") < strings.Join(b.Path, ".")
-}
