@@ -15,4 +15,8 @@ type TestSuiteRunner interface {
 	Test() (Status, tfdiags.Diagnostics)
 	Stop()
 	Cancel()
+
+	// IsStopped allows code outside the moduletest package to confirm the suite was stopped
+	// when handling a graceful exit scenario
+	IsStopped() bool
 }
