@@ -15,7 +15,7 @@ func TestProviderAddrs(t *testing.T) {
 
 	plan := &Plan{
 		VariableValues: map[string]DynamicValue{},
-		Changes: &Changes{
+		Changes: &ChangesSrc{
 			Resources: []*ResourceInstanceChangeSrc{
 				{
 					Addr: addrs.Resource{
@@ -74,7 +74,7 @@ func TestProviderAddrs(t *testing.T) {
 
 // Module outputs should not effect the result of Empty
 func TestModuleOutputChangesEmpty(t *testing.T) {
-	changes := &Changes{
+	changes := &ChangesSrc{
 		Outputs: []*OutputChangeSrc{
 			{
 				Addr: addrs.AbsOutputValue{
