@@ -7,7 +7,6 @@ import (
 	"github.com/zclconf/go-cty/cty"
 
 	"github.com/hashicorp/terraform/internal/configs/configschema"
-	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
@@ -17,6 +16,11 @@ type Interface interface {
 	// GetSchema returns the complete schema for the provider.
 	GetProviderSchema() GetProviderSchemaResponse
 
+	// GetResourceIdentitySchemas returns the identity schemas for all managed resources
+	// for the provider. Usually you don't need to call this method directly as GetProviderSchema
+	// will merge the identity schemas into the provider schema.
+	GetResourceIdentitySchemas() GetResourceIdentitySchemasResponse
+
 	// ValidateProviderConfig allows the provider to validate the configuration.
 	// The ValidateProviderConfigResponse.PreparedConfig field is unused. The
 	// final configuration is not stored in the state, and any modifications
@@ -41,6 +45,12 @@ type Interface interface {
 	// result is used for any further processing.
 	UpgradeResourceState(UpgradeResourceStateRequest) UpgradeResourceStateResponse
 
+	// UpgradeResourceIdentity is called when the state loader encounters an
+	// instance identity whose schema version is less than the one reported by
+	// the currently-used version of the corresponding provider, and the upgraded
+	// result is used for any further processing.
+	UpgradeResourceIdentity(UpgradeResourceIdentityRequest) UpgradeResourceIdentityResponse
+
 	// Configure configures and initialized the provider.
 	ConfigureProvider(ConfigureProviderRequest) ConfigureProviderResponse
 
@@ -98,7 +108,7 @@ type Interface interface {
 // should only be used when handling a value for that method. The handling of
 // of schemas in any other context should always use ProviderSchema, so that
 // the in-memory representation can be more easily changed separately from the
-// RCP protocol.
+// RPC protocol.
 type GetProviderSchemaResponse struct {
 	// Provider is the schema for the provider itself.
 	Provider Schema
@@ -127,6 +137,25 @@ type GetProviderSchemaResponse struct {
 	ServerCapabilities ServerCapabilities
 }
 
+// GetResourceIdentitySchemasResponse is the return type for GetResourceIdentitySchemas,
+// and should only be used when handling a value for that method. The handling of
+// of schemas in any other context should always use ResourceIdentitySchemas, so that
+// the in-memory representation can be more easily changed separately from the
+// RPC protocol.
+type GetResourceIdentitySchemasResponse struct {
+	// IdentityTypes map the resource type name to that type's identity schema.
+	IdentityTypes map[string]IdentitySchema
+
+	// Diagnostics contains any warnings or errors from the method call.
+	Diagnostics tfdiags.Diagnostics
+}
+
+type IdentitySchema struct {
+	Version int64
+
+	Body *configschema.Object
+}
+
 // Schema pairs a provider or resource schema with that schema's version.
 // This is used to be able to upgrade the schema in UpgradeResourceState.
 //
@@ -135,7 +164,10 @@ type GetProviderSchemaResponse struct {
 // for everything within a particular provider.
 type Schema struct {
 	Version int64
-	Block   *configschema.Block
+	Body    *configschema.Block
+
+	IdentityVersion int64
+	Identity        *configschema.Object
 }
 
 // ServerCapabilities allows providers to communicate extra information
@@ -254,6 +286,26 @@ type UpgradeResourceStateResponse struct {
 	Diagnostics tfdiags.Diagnostics
 }
 
+type UpgradeResourceIdentityRequest struct {
+	// TypeName is the name of the resource type being upgraded
+	TypeName string
+
+	// Version is version of the schema that created the current identity.
+	Version int64
+
+	// RawIdentityJSON contains the identity that needs to be
+	// upgraded to match the current schema version.
+	RawIdentityJSON []byte
+}
+
+type UpgradeResourceIdentityResponse struct {
+	// UpgradedState is the newly upgraded resource identity.
+	UpgradedIdentity cty.Value
+
+	// Diagnostics contains any warnings or errors from the method call.
+	Diagnostics tfdiags.Diagnostics
+}
+
 type ConfigureProviderRequest struct {
 	// Terraform version is the version string from the running instance of
 	// terraform. Providers can use TerraformVersion to verify compatibility,
@@ -291,6 +343,9 @@ type ReadResourceRequest struct {
 
 	// ClientCapabilities contains information about the client's capabilities.
 	ClientCapabilities ClientCapabilities
+
+	// CurrentIdentity is the current identity data of the resource.
+	CurrentIdentity cty.Value
 }
 
 // DeferredReason is a string that describes why a resource was deferred.
@@ -344,6 +399,10 @@ type ReadResourceResponse struct {
 	// Deferred if present signals that the provider was not able to fully
 	// complete this operation and a susequent run is required.
 	Deferred *Deferred
+
+	// Identity is the object-typed value representing the identity of the remote
+	// object within Terraform.
+	Identity cty.Value
 }
 
 type PlanResourceChangeRequest struct {
@@ -377,6 +436,9 @@ type PlanResourceChangeRequest struct {
 
 	// ClientCapabilities contains information about the client's capabilities.
 	ClientCapabilities ClientCapabilities
+
+	// PriorIdentity is the current identity data of the resource.
+	PriorIdentity cty.Value
 }
 
 type PlanResourceChangeResponse struct {
@@ -406,6 +468,9 @@ type PlanResourceChangeResponse struct {
 	// Deferred if present signals that the provider was not able to fully
 	// complete this operation and a susequent run is required.
 	Deferred *Deferred
+
+	// PlannedIdentity is the planned identity data of the resource.
+	PlannedIdentity cty.Value
 }
 
 type ApplyResourceChangeRequest struct {
@@ -433,6 +498,9 @@ type ApplyResourceChangeRequest struct {
 	// each provider, and it should not be used without coordination with
 	// HashiCorp. It is considered experimental and subject to change.
 	ProviderMeta cty.Value
+
+	// PlannedIdentity is the planned identity data of the resource.
+	PlannedIdentity cty.Value
 }
 
 type ApplyResourceChangeResponse struct {
@@ -454,6 +522,9 @@ type ApplyResourceChangeResponse struct {
 	// otherwise fail due to this imprecise mapping. No other provider or SDK
 	// implementation is permitted to set this.
 	LegacyTypeSystem bool
+
+	// NewIdentity is the new identity data of the resource.
+	NewIdentity cty.Value
 }
 
 type ImportResourceStateRequest struct {
@@ -466,6 +537,9 @@ type ImportResourceStateRequest struct {
 
 	// ClientCapabilities contains information about the client's capabilities.
 	ClientCapabilities ClientCapabilities
+
+	// Identity is the identity data of the resource.
+	Identity cty.Value
 }
 
 type ImportResourceStateResponse struct {
@@ -484,7 +558,7 @@ type ImportResourceStateResponse struct {
 }
 
 // ImportedResource represents an object being imported into Terraform with the
-// help of a provider. An ImportedObject is a RemoteObject that has been read
+// help of a provider. An ImportedResource is a RemoteObject that has been read
 // by the provider's import handler but hasn't yet been committed to state.
 type ImportedResource struct {
 	// TypeName is the name of the resource type associated with the
@@ -500,6 +574,9 @@ type ImportedResource struct {
 	// Private is an opaque blob that will be stored in state along with the
 	// resource. It is intended only for interpretation by the provider itself.
 	Private []byte
+
+	// Identity is the identity data of the resource.
+	Identity cty.Value
 }
 
 type MoveResourceStateRequest struct {
@@ -527,6 +604,9 @@ type MoveResourceStateRequest struct {
 	// TargetTypeName is the name of the resource type that the resource is
 	// being moved to.
 	TargetTypeName string
+
+	// SourceIdentity is the identity data of the resource that is being moved.
+	SourceIdentity []byte
 }
 
 type MoveResourceStateResponse struct {
@@ -540,24 +620,9 @@ type MoveResourceStateResponse struct {
 
 	// Diagnostics contains any warnings or errors from the method call.
 	Diagnostics tfdiags.Diagnostics
-}
 
-// AsInstanceObject converts the receiving ImportedObject into a
-// ResourceInstanceObject that has status ObjectReady.
-//
-// The returned object does not know its own resource type, so the caller must
-// retain the ResourceType value from the source object if this information is
-// needed.
-//
-// The returned object also has no dependency addresses, but the caller may
-// freely modify the direct fields of the returned object without affecting
-// the receiver.
-func (ir ImportedResource) AsInstanceObject() *states.ResourceInstanceObject {
-	return &states.ResourceInstanceObject{
-		Status:  states.ObjectReady,
-		Value:   ir.State,
-		Private: ir.Private,
-	}
+	// TargetIdentity is the identity data of the resource that is being moved.
+	TargetIdentity cty.Value
 }
 
 type ReadDataSourceRequest struct {
