@@ -14,6 +14,7 @@ import (
 	"go.opentelemetry.io/otel/trace"
 	"google.golang.org/protobuf/types/known/anypb"
 
+	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/collections"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/promising"
@@ -91,7 +92,7 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 						log.Printf("[TRACE] stackeval: %s preparing to apply", addr)
 
 						stack := main.Stack(ctx, addr.Stack, ApplyPhase)
-						component, removed := stack.ApplyableComponents(ctx, addr.Item.Component)
+						component, removed := stack.ApplyableComponents(addr.Item.Component)
 
 						// A component change can be sourced from a removed
 						// block or a component block. We'll try to find the
@@ -99,66 +100,53 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 
 						var inst ApplyableComponentInstance
 
-						if removed != nil {
-							if insts, unknown, _ := removed.Instances(ctx, ApplyPhase); unknown {
-								// It might be that either the removed block
-								// or component block was deferred but the
-								// other one had proper changes. We'll note
-								// this in the logs but just skip processing
-								// it.
-								log.Printf("[TRACE]: %s has planned changes, but was unknown. Check further messages to find out if this was an error.", addr)
+						matchedUnknownBlock := false
+
+					Blocks:
+						for _, block := range removed {
+							if insts, unknown, _ := block.Instances(ctx, ApplyPhase); unknown {
+								matchedUnknownBlock = true
 							} else {
-								i, ok := insts[addr.Item.Key]
-								if !ok {
-									// Again, this might be okay if the component
-									// block was deferred but the removed block had
-									// proper changes (or vice versa). We'll note
-									// this in the logs but just skip processing it.
-									log.Printf("[TRACE]: %s has planned changes, but does not seem to be declared. Check further messages to find out if this was an error.", addr)
-								} else {
-									inst = i
+								for _, i := range insts {
+									if i.from.Item.Key == addr.Item.Key {
+										inst = i
+										break Blocks
+									}
 								}
 							}
 						}
 
 						if component != nil {
 							if insts, unknown := component.Instances(ctx, ApplyPhase); unknown {
-								// It might be that either the removed block
-								// or component block was deferred but the
-								// other one had proper changes. We'll note
-								// this in the logs but just skip processing
-								// it.
-								log.Printf("[TRACE]: %s has planned changes, but was unknown. Check further messages to find out if this was an error.", addr)
+								matchedUnknownBlock = true
 							} else {
-								if i, ok := insts[addr.Item.Key]; !ok {
-									// Again, this might be okay if the component
-									// block was deferred but the removed block had
-									// proper changes (or vice versa). We'll note
-									// this in the logs but just skip processing it.
-									log.Printf("[TRACE]: %s has planned changes, but does not seem to be declared. Check further messages to find out if this was an error.", addr)
-								} else {
+								if i, ok := insts[addr.Item.Key]; ok {
 									if inst != nil {
-										// Problem! We have both a removed block and
-										// a component instance that point to the same
-										// address. This should not happen. The plan
-										// should have caught this and resulted in an
-										// unapplyable plan.
+										var diags tfdiags.Diagnostics
+										diags = diags.Append(tfdiags.Sourceless(
+											tfdiags.Error,
+											"Invalid component instance",
+											fmt.Sprintf("There was a plan for %s, which matched both removed and component blocks.", addr),
+										))
 										log.Printf("[ERROR] stackeval: %s has both a component and a removed block that point to the same address", addr)
 										span.SetStatus(codes.Error, "both component and removed block present")
-										return nil, nil
+										return nil, diags
 									}
 									inst = i
 								}
 							}
 						}
 
-						if inst == nil {
-							// Then we have a problem. We have a component
-							// that has planned changes but no instance to
-							// apply them to. This should not happen.
+						if inst == nil && !matchedUnknownBlock {
+							var diags tfdiags.Diagnostics
+							diags = diags.Append(tfdiags.Sourceless(
+								tfdiags.Error,
+								"Invalid component instance",
+								fmt.Sprintf("There was a plan for %s, which matched no known component instances.", addr),
+							))
 							log.Printf("[ERROR] stackeval: %s has planned changes, but no instance to apply them to", addr)
 							span.SetStatus(codes.Error, "no instance to apply changes to")
-							return nil, nil
+							return nil, diags
 						}
 
 						modulesRuntimePlan, err := componentInstPlan.ForModulesRuntime()
@@ -180,6 +168,34 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 							return nil, diags
 						}
 
+						if matchedUnknownBlock && inst == nil {
+							log.Printf("[TRACE] stackeval: %s matched an unknown block so returning prior state unchanged.", addr)
+							span.SetStatus(codes.Ok, "apply unknown")
+
+							for _, step := range addr.Stack {
+								if step.Key == addrs.WildcardKey {
+									// for instances that were unknown we don't
+									// emit any updates for them.
+									return nil, nil
+								}
+							}
+							if addr.Item.Key == addrs.WildcardKey {
+								// for instances that were unknown we don't
+								// emit any updates for them.
+								return nil, nil
+							}
+
+							// if we get here, then this was a concrete instance
+							// that was in state before the original plan, we
+							// do want to emit a "nothing changed" update for
+							// this instance.
+
+							return &ComponentInstanceApplyResult{
+								FinalState: modulesRuntimePlan.PrevRunState,
+								Complete:   false,
+							}, nil
+						}
+
 						var waitForComponents collections.Set[stackaddrs.AbsComponent]
 						var waitForRemoveds collections.Set[stackaddrs.AbsComponent]
 						if action == plans.Delete || action == plans.Forget {
@@ -208,7 +224,7 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 						}
 						for waitComponentAddr := range waitForComponents.All() {
 							if stack := main.Stack(ctx, waitComponentAddr.Stack, ApplyPhase); stack != nil {
-								if component := stack.Component(ctx, waitComponentAddr.Item); component != nil {
+								if component := stack.Component(waitComponentAddr.Item); component != nil {
 									span.AddEvent("awaiting predecessor", trace.WithAttributes(
 										attribute.String("component_addr", waitComponentAddr.String()),
 									))
@@ -224,7 +240,7 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 
 										// We'll return a stub result that reports that nothing was changed, since
 										// we're not going to run our apply phase at all.
-										return inst.PlaceholderApplyResultForSkippedApply(ctx, modulesRuntimePlan), nil
+										return inst.PlaceholderApplyResultForSkippedApply(modulesRuntimePlan), nil
 										// Since we're not calling inst.ApplyModuleTreePlan at all in this
 										// codepath, the stacks runtime will not emit any progress events for
 										// this component instance or any of the objects inside it.
@@ -234,11 +250,16 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 						}
 						for waitComponentAddr := range waitForRemoveds.All() {
 							if stack := main.Stack(ctx, waitComponentAddr.Stack, ApplyPhase); stack != nil {
-								if removed := stack.Removed(ctx, waitComponentAddr.Item); removed != nil {
+								if removed := stack.RemovedComponent(waitComponentAddr.Item); removed != nil {
 									span.AddEvent("awaiting predecessor", trace.WithAttributes(
 										attribute.String("component_addr", waitComponentAddr.String()),
 									))
-									success := removed.ApplySuccessful(ctx)
+									success := true
+									for _, block := range removed {
+										if !block.ApplySuccessful(ctx) {
+											success = false
+										}
+									}
 									if !success {
 										// If anything we're waiting on does not succeed then we can't proceed without
 										// violating the dependency invariants.
@@ -250,7 +271,7 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 
 										// We'll return a stub result that reports that nothing was changed, since
 										// we're not going to run our apply phase at all.
-										return inst.PlaceholderApplyResultForSkippedApply(ctx, modulesRuntimePlan), nil
+										return inst.PlaceholderApplyResultForSkippedApply(modulesRuntimePlan), nil
 										// Since we're not calling inst.ApplyModuleTreePlan at all in this
 										// codepath, the stacks runtime will not emit any progress events for
 										// this component instance or any of the objects inside it.
@@ -338,7 +359,7 @@ func ApplyPlan(ctx context.Context, config *stackconfig.Config, plan *stackplan.
 	})
 	diags := withDiags.Diagnostics
 	main := withDiags.Result
-	diags = diags.Append(diagnosticsForPromisingTaskError(err, main))
+	diags = diags.Append(diagnosticsForPromisingTaskError(err))
 	if len(diags) > 0 {
 		outp.AnnounceDiagnostics(ctx, diags)
 	}
