@@ -171,11 +171,11 @@ func TestPlanning_DestroyMode(t *testing.T) {
 				return &providerTesting.MockProvider{
 					GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 						Provider: providers.Schema{
-							Block: &configschema.Block{},
+							Body: &configschema.Block{},
 						},
 						ResourceTypes: map[string]providers.Schema{
 							"test": {
-								Block: resourceTypeSchema,
+								Body: resourceTypeSchema,
 							},
 						},
 						ServerCapabilities: providers.ServerCapabilities{
@@ -271,7 +271,9 @@ func TestPlanning_DestroyMode(t *testing.T) {
 
 	if planSrc := aPlan.Changes.ResourceInstance(aResourceInstAddr.Item); planSrc != nil {
 		rAddr := aResourceInstAddr
-		plan, err := planSrc.Decode(resourceTypeSchema.ImpliedType())
+		plan, err := planSrc.Decode(providers.Schema{
+			Body: resourceTypeSchema,
+		})
 		if err != nil {
 			t.Fatalf("can't decode change for %s: %s", rAddr, err)
 		}
@@ -295,7 +297,9 @@ func TestPlanning_DestroyMode(t *testing.T) {
 
 	if planSrc := bPlan.Changes.ResourceInstance(bResourceInstAddr.Item); planSrc != nil {
 		rAddr := bResourceInstAddr
-		plan, err := planSrc.Decode(resourceTypeSchema.ImpliedType())
+		plan, err := planSrc.Decode(providers.Schema{
+			Body: resourceTypeSchema,
+		})
 		if err != nil {
 			t.Fatalf("can't decode change for %s: %s", rAddr, err)
 		}
@@ -358,7 +362,7 @@ func TestPlanning_RequiredComponents(t *testing.T) {
 				return &providerTesting.MockProvider{
 					GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 						Provider: providers.Schema{
-							Block: &configschema.Block{
+							Body: &configschema.Block{
 								Attributes: map[string]*configschema.Attribute{
 									"in": {
 										Type:     cty.Map(cty.String),
@@ -502,7 +506,7 @@ func TestPlanning_RequiredComponents(t *testing.T) {
 					if stack == nil {
 						t.Fatalf("no declaration for %s", test.componentAddr.Stack)
 					}
-					component := stack.Component(ctx, test.componentAddr.Item)
+					component := stack.Component(test.componentAddr.Item)
 					if component == nil {
 						t.Fatalf("no declaration for %s", test.componentAddr)
 					}
@@ -545,11 +549,11 @@ func TestPlanning_DeferredChangesPropagation(t *testing.T) {
 				return &providerTesting.MockProvider{
 					GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 						Provider: providers.Schema{
-							Block: &configschema.Block{},
+							Body: &configschema.Block{},
 						},
 						ResourceTypes: map[string]providers.Schema{
 							"test": {
-								Block: &configschema.Block{},
+								Body: &configschema.Block{},
 							},
 						},
 					},
@@ -635,7 +639,7 @@ func TestPlanning_RemoveDataResource(t *testing.T) {
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
 						"test": {
-							Block: &configschema.Block{},
+							Body: &configschema.Block{},
 						},
 					},
 				},
