@@ -15,7 +15,6 @@ import (
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/instances"
 	"github.com/hashicorp/terraform/internal/lang"
-	"github.com/hashicorp/terraform/internal/promising"
 	"github.com/hashicorp/terraform/internal/stacks/stackaddrs"
 	"github.com/hashicorp/terraform/internal/stacks/stackconfig"
 	"github.com/hashicorp/terraform/internal/tfdiags"
@@ -31,6 +30,7 @@ type StackConfig struct {
 	addr stackaddrs.Stack
 
 	config *stackconfig.ConfigNode
+	parent *StackConfig
 
 	main *Main
 
@@ -43,18 +43,18 @@ type StackConfig struct {
 	outputValues   map[stackaddrs.OutputValue]*OutputValueConfig
 	stackCalls     map[stackaddrs.StackCall]*StackCallConfig
 	components     map[stackaddrs.Component]*ComponentConfig
-	removed        map[stackaddrs.Component]*RemovedConfig
+	removed        map[stackaddrs.Component][]*RemovedComponentConfig
 	providers      map[stackaddrs.ProviderConfig]*ProviderConfig
 }
 
 var (
-	_ ExpressionScope      = (*StackConfig)(nil)
-	_ namedPromiseReporter = (*StackConfig)(nil)
+	_ ExpressionScope = (*StackConfig)(nil)
 )
 
-func newStackConfig(main *Main, addr stackaddrs.Stack, config *stackconfig.ConfigNode) *StackConfig {
+func newStackConfig(main *Main, addr stackaddrs.Stack, parent *StackConfig, config *stackconfig.ConfigNode) *StackConfig {
 	return &StackConfig{
 		addr:   addr,
+		parent: parent,
 		config: config,
 		main:   main,
 
@@ -64,48 +64,14 @@ func newStackConfig(main *Main, addr stackaddrs.Stack, config *stackconfig.Confi
 		outputValues:   make(map[stackaddrs.OutputValue]*OutputValueConfig, len(config.Stack.Declarations.OutputValues)),
 		stackCalls:     make(map[stackaddrs.StackCall]*StackCallConfig, len(config.Stack.Declarations.EmbeddedStacks)),
 		components:     make(map[stackaddrs.Component]*ComponentConfig, len(config.Stack.Declarations.Components)),
-		removed:        make(map[stackaddrs.Component]*RemovedConfig, len(config.Stack.Declarations.Removed)),
+		removed:        make(map[stackaddrs.Component][]*RemovedComponentConfig, len(config.Stack.Declarations.Removed)),
 		providers:      make(map[stackaddrs.ProviderConfig]*ProviderConfig, len(config.Stack.Declarations.ProviderConfigs)),
 	}
 }
 
-func (s *StackConfig) Addr() stackaddrs.Stack {
-	return s.addr
-}
-
-func (s *StackConfig) IsRoot() bool {
-	return s.addr.IsRoot()
-}
-
-// ParentAddr returns the address of the containing stack, or panics if called
-// on the root stack (since it has no parent).
-func (s *StackConfig) ParentAddr() stackaddrs.Stack {
-	return s.addr.Parent()
-}
-
-// ConfigDeclarations returns a pointer to the [stackconfig.Declarations]
-// object describing the configured declarations from this stack config's
-// configuration files.
-func (s *StackConfig) ConfigDeclarations(ctx context.Context) *stackconfig.Declarations {
-	return &s.config.Stack.Declarations
-}
-
-// ParentConfig returns the [StackConfig] object representing the configuration
-// of the containing stack, or nil if the receiver is the root stack in the
-// tree.
-func (s *StackConfig) ParentConfig(ctx context.Context) *StackConfig {
-	if s.IsRoot() {
-		return nil
-	}
-	// Each StackConfig is only responsible for looking after its direct
-	// children, so to navigate upwards we need to start back at the
-	// root and work our way through the tree again.
-	return s.main.StackConfig(ctx, s.ParentAddr())
-}
-
 // ChildConfig returns a [StackConfig] representing the embedded stack matching
 // the given address step, or nil if there is no such stack.
-func (s *StackConfig) ChildConfig(ctx context.Context, step stackaddrs.StackStep) *StackConfig {
+func (s *StackConfig) ChildConfig(step stackaddrs.StackStep) *StackConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -115,35 +81,35 @@ func (s *StackConfig) ChildConfig(ctx context.Context, step stackaddrs.StackStep
 		if !ok {
 			return nil
 		}
-		childAddr := s.Addr().Child(step.Name)
-		s.children[step] = newStackConfig(s.main, childAddr, childNode)
+		childAddr := s.addr.Child(step.Name)
+		s.children[step] = newStackConfig(s.main, childAddr, s, childNode)
 		ret = s.children[step]
 	}
 	return ret
 }
 
-func (s *StackConfig) ChildConfigs(ctx context.Context) map[stackaddrs.StackStep]*StackConfig {
+func (s *StackConfig) ChildConfigs() map[stackaddrs.StackStep]*StackConfig {
 	if len(s.config.Children) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.StackStep]*StackConfig, len(s.config.Children))
 	for n := range s.config.Children {
 		stepAddr := stackaddrs.StackStep{Name: n}
-		ret[stepAddr] = s.ChildConfig(ctx, stepAddr)
+		ret[stepAddr] = s.ChildConfig(stepAddr)
 	}
 	return ret
 }
 
 // InputVariables returns a map of the objects representing all of the
 // input variables declared inside this stack configuration.
-func (s *StackConfig) InputVariables(ctx context.Context) map[stackaddrs.InputVariable]*InputVariableConfig {
+func (s *StackConfig) InputVariables() map[stackaddrs.InputVariable]*InputVariableConfig {
 	if len(s.config.Stack.InputVariables) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.InputVariable]*InputVariableConfig, len(s.config.Stack.InputVariables))
 	for name := range s.config.Stack.InputVariables {
 		addr := stackaddrs.InputVariable{Name: name}
-		ret[addr] = s.InputVariable(ctx, addr)
+		ret[addr] = s.InputVariable(addr)
 	}
 	return ret
 }
@@ -151,7 +117,7 @@ func (s *StackConfig) InputVariables(ctx context.Context) map[stackaddrs.InputVa
 // InputVariable returns an [InputVariableConfig] representing the input
 // variable declared within this stack config that matches the given
 // address, or nil if there is no such declaration.
-func (s *StackConfig) InputVariable(ctx context.Context, addr stackaddrs.InputVariable) *InputVariableConfig {
+func (s *StackConfig) InputVariable(addr stackaddrs.InputVariable) *InputVariableConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -161,23 +127,23 @@ func (s *StackConfig) InputVariable(ctx context.Context, addr stackaddrs.InputVa
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newInputVariableConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newInputVariableConfig(s.main, cfgAddr, s, cfg)
 		s.inputVariables[addr] = ret
 	}
 	return ret
 }
 
-// InputVariables returns a map of the objects representing all of the
+// LocalValues returns a map of the objects representing all of the
 // local values declared inside this stack configuration.
-func (s *StackConfig) LocalValues(ctx context.Context) map[stackaddrs.LocalValue]*LocalValueConfig {
+func (s *StackConfig) LocalValues() map[stackaddrs.LocalValue]*LocalValueConfig {
 	if len(s.config.Stack.LocalValues) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.LocalValue]*LocalValueConfig, len(s.config.Stack.LocalValues))
 	for name := range s.config.Stack.LocalValues {
 		addr := stackaddrs.LocalValue{Name: name}
-		ret[addr] = s.LocalValue(ctx, addr)
+		ret[addr] = s.LocalValue(addr)
 	}
 	return ret
 }
@@ -185,7 +151,7 @@ func (s *StackConfig) LocalValues(ctx context.Context) map[stackaddrs.LocalValue
 // LocalValue returns an [LocalValueConfig] representing the input
 // variable declared within this stack config that matches the given
 // address, or nil if there is no such declaration.
-func (s *StackConfig) LocalValue(_ context.Context, addr stackaddrs.LocalValue) *LocalValueConfig {
+func (s *StackConfig) LocalValue(addr stackaddrs.LocalValue) *LocalValueConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -195,8 +161,8 @@ func (s *StackConfig) LocalValue(_ context.Context, addr stackaddrs.LocalValue)
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newLocalValueConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newLocalValueConfig(s.main, cfgAddr, s, cfg)
 		s.localValues[addr] = ret
 	}
 	return ret
@@ -205,7 +171,7 @@ func (s *StackConfig) LocalValue(_ context.Context, addr stackaddrs.LocalValue)
 // OutputValue returns an [OutputValueConfig] representing the output
 // value declared within this stack config that matches the given
 // address, or nil if there is no such declaration.
-func (s *StackConfig) OutputValue(ctx context.Context, addr stackaddrs.OutputValue) *OutputValueConfig {
+func (s *StackConfig) OutputValue(addr stackaddrs.OutputValue) *OutputValueConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -215,8 +181,8 @@ func (s *StackConfig) OutputValue(ctx context.Context, addr stackaddrs.OutputVal
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newOutputValueConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newOutputValueConfig(s.main, cfgAddr, s, cfg)
 		s.outputValues[addr] = ret
 	}
 	return ret
@@ -224,14 +190,14 @@ func (s *StackConfig) OutputValue(ctx context.Context, addr stackaddrs.OutputVal
 
 // OutputValues returns a map of the objects representing all of the
 // output values declared inside this stack configuration.
-func (s *StackConfig) OutputValues(ctx context.Context) map[stackaddrs.OutputValue]*OutputValueConfig {
+func (s *StackConfig) OutputValues() map[stackaddrs.OutputValue]*OutputValueConfig {
 	if len(s.config.Stack.OutputValues) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.OutputValue]*OutputValueConfig, len(s.config.Stack.OutputValues))
 	for name := range s.config.Stack.OutputValues {
 		addr := stackaddrs.OutputValue{Name: name}
-		ret[addr] = s.OutputValue(ctx, addr)
+		ret[addr] = s.OutputValue(addr)
 	}
 	return ret
 }
@@ -239,18 +205,18 @@ func (s *StackConfig) OutputValues(ctx context.Context) map[stackaddrs.OutputVal
 // ResultType returns the type of the result object that will be produced
 // by this stack configuration, based on the output values declared within
 // it.
-func (s *StackConfig) ResultType(ctx context.Context) cty.Type {
-	os := s.OutputValues(ctx)
+func (s *StackConfig) ResultType() cty.Type {
+	os := s.OutputValues()
 	atys := make(map[string]cty.Type, len(os))
 	for addr, o := range os {
-		atys[addr.Name] = o.ValueTypeConstraint(ctx)
+		atys[addr.Name] = o.ValueTypeConstraint()
 	}
 	return cty.Object(atys)
 }
 
 // Providers returns a map of the objects representing all of the provider
 // configurations declared inside this stack configuration.
-func (s *StackConfig) Providers(ctx context.Context) map[stackaddrs.ProviderConfig]*ProviderConfig {
+func (s *StackConfig) Providers() map[stackaddrs.ProviderConfig]*ProviderConfig {
 	if len(s.config.Stack.ProviderConfigs) == 0 {
 		return nil
 	}
@@ -267,7 +233,7 @@ func (s *StackConfig) Providers(ctx context.Context) map[stackaddrs.ProviderConf
 			Provider: provider,
 			Name:     configAddr.Alias,
 		}
-		ret[addr] = s.Provider(ctx, addr)
+		ret[addr] = s.Provider(addr)
 	}
 	return ret
 }
@@ -275,7 +241,7 @@ func (s *StackConfig) Providers(ctx context.Context) map[stackaddrs.ProviderConf
 // Provider returns a [ProviderConfig] representing the provider configuration
 // block within the stack configuration that matches the given address,
 // or nil if there is no such declaration.
-func (s *StackConfig) Provider(ctx context.Context, addr stackaddrs.ProviderConfig) *ProviderConfig {
+func (s *StackConfig) Provider(addr stackaddrs.ProviderConfig) *ProviderConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -295,8 +261,8 @@ func (s *StackConfig) Provider(ctx context.Context, addr stackaddrs.ProviderConf
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newProviderConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newProviderConfig(s.main, cfgAddr, s, cfg)
 		s.providers[addr] = ret
 	}
 	return ret
@@ -308,7 +274,7 @@ func (s *StackConfig) Provider(ctx context.Context, addr stackaddrs.ProviderConf
 //
 // This is equivalent to calling [Provider] just using a reference address
 // instead of a config address.
-func (s *StackConfig) ProviderByLocalAddr(ctx context.Context, localAddr stackaddrs.ProviderConfigRef) *ProviderConfig {
+func (s *StackConfig) ProviderByLocalAddr(localAddr stackaddrs.ProviderConfigRef) *ProviderConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -331,8 +297,8 @@ func (s *StackConfig) ProviderByLocalAddr(ctx context.Context, localAddr stackad
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newProviderConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newProviderConfig(s.main, cfgAddr, s, cfg)
 		s.providers[addr] = ret
 	}
 	return ret
@@ -344,7 +310,7 @@ func (s *StackConfig) ProviderByLocalAddr(ctx context.Context, localAddr stackad
 //
 // If the second return value is false then there is no local name declared
 // for the given provider, and so the first return value is invalid.
-func (s *StackConfig) ProviderLocalName(ctx context.Context, addr addrs.Provider) (string, bool) {
+func (s *StackConfig) ProviderLocalName(addr addrs.Provider) (string, bool) {
 	return s.config.Stack.RequiredProviders.LocalNameForProvider(addr)
 }
 
@@ -354,14 +320,14 @@ func (s *StackConfig) ProviderLocalName(ctx context.Context, addr addrs.Provider
 //
 // If the second return value is false then there is no provider declared
 // for the given local name, and so the first return value is invalid.
-func (s *StackConfig) ProviderForLocalName(ctx context.Context, localName string) (addrs.Provider, bool) {
+func (s *StackConfig) ProviderForLocalName(localName string) (addrs.Provider, bool) {
 	return s.config.Stack.RequiredProviders.ProviderForLocalName(localName)
 }
 
 // StackCall returns a [StackCallConfig] representing the "stack" block
 // matching the given address declared within this stack config, or nil if
 // there is no such declaration.
-func (s *StackConfig) StackCall(ctx context.Context, addr stackaddrs.StackCall) *StackCallConfig {
+func (s *StackConfig) StackCall(addr stackaddrs.StackCall) *StackCallConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -371,8 +337,8 @@ func (s *StackConfig) StackCall(ctx context.Context, addr stackaddrs.StackCall)
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newStackCallConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newStackCallConfig(s.main, cfgAddr, s, cfg)
 		s.stackCalls[addr] = ret
 	}
 	return ret
@@ -380,14 +346,14 @@ func (s *StackConfig) StackCall(ctx context.Context, addr stackaddrs.StackCall)
 
 // StackCalls returns a map of objects representing all of the embedded stack
 // calls inside this stack configuration.
-func (s *StackConfig) StackCalls(ctx context.Context) map[stackaddrs.StackCall]*StackCallConfig {
+func (s *StackConfig) StackCalls() map[stackaddrs.StackCall]*StackCallConfig {
 	if len(s.config.Children) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.StackCall]*StackCallConfig, len(s.config.Children))
 	for n := range s.config.Children {
 		stepAddr := stackaddrs.StackCall{Name: n}
-		ret[stepAddr] = s.StackCall(ctx, stepAddr)
+		ret[stepAddr] = s.StackCall(stepAddr)
 	}
 	return ret
 }
@@ -395,7 +361,7 @@ func (s *StackConfig) StackCalls(ctx context.Context) map[stackaddrs.StackCall]*
 // Component returns a [ComponentConfig] representing the component call
 // declared within this stack config that matches the given address, or nil if
 // there is no such declaration.
-func (s *StackConfig) Component(ctx context.Context, addr stackaddrs.Component) *ComponentConfig {
+func (s *StackConfig) Component(addr stackaddrs.Component) *ComponentConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -405,8 +371,8 @@ func (s *StackConfig) Component(ctx context.Context, addr stackaddrs.Component)
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newComponentConfig(s.main, cfgAddr, cfg)
+		cfgAddr := stackaddrs.Config(s.addr, addr)
+		ret = newComponentConfig(s.main, cfgAddr, s, cfg)
 		s.components[addr] = ret
 	}
 	return ret
@@ -414,48 +380,51 @@ func (s *StackConfig) Component(ctx context.Context, addr stackaddrs.Component)
 
 // Components returns a map of the objects representing all of the
 // component calls declared inside this stack configuration.
-func (s *StackConfig) Components(ctx context.Context) map[stackaddrs.Component]*ComponentConfig {
+func (s *StackConfig) Components() map[stackaddrs.Component]*ComponentConfig {
 	if len(s.config.Stack.Components) == 0 {
 		return nil
 	}
 	ret := make(map[stackaddrs.Component]*ComponentConfig, len(s.config.Stack.Components))
 	for name := range s.config.Stack.Components {
 		addr := stackaddrs.Component{Name: name}
-		ret[addr] = s.Component(ctx, addr)
+		ret[addr] = s.Component(addr)
 	}
 	return ret
 }
 
-// Removed returns a [RemovedConfig] representing the component call
-// declared within this stack config that matches the given address, or nil if
-// there is no such declaration.
-func (s *StackConfig) Removed(ctx context.Context, addr stackaddrs.Component) *RemovedConfig {
+// RemovedComponent returns a [RemovedComponentConfig] representing the
+// component call declared within this stack config that matches the given
+// address, or nil if there is no such declaration.
+func (s *StackConfig) RemovedComponent(addr stackaddrs.Component) []*RemovedComponentConfig {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
 	ret, ok := s.removed[addr]
 	if !ok {
-		cfg, ok := s.config.Stack.Removed[addr.Name]
+		cfgs, ok := s.config.Stack.Removed[addr.Name]
 		if !ok {
 			return nil
 		}
-		cfgAddr := stackaddrs.Config(s.Addr(), addr)
-		ret = newRemovedConfig(s.main, cfgAddr, cfg)
+		for _, cfg := range cfgs {
+			cfgAddr := stackaddrs.Config(s.addr, addr)
+			removed := newRemovedComponentConfig(s.main, cfgAddr, s, cfg)
+			ret = append(ret, removed)
+		}
 		s.removed[addr] = ret
 	}
 	return ret
 }
 
-// Removeds returns a map of the objects representing all of the
+// RemovedComponents returns a map of the objects representing all of the
 // removed calls declared inside this stack configuration.
-func (s *StackConfig) Removeds(ctx context.Context) map[stackaddrs.Component]*RemovedConfig {
+func (s *StackConfig) RemovedComponents() map[stackaddrs.Component][]*RemovedComponentConfig {
 	if len(s.config.Stack.Removed) == 0 {
 		return nil
 	}
-	ret := make(map[stackaddrs.Component]*RemovedConfig, len(s.config.Stack.Removed))
+	ret := make(map[stackaddrs.Component][]*RemovedComponentConfig, len(s.config.Stack.Removed))
 	for name := range s.config.Stack.Removed {
 		addr := stackaddrs.Component{Name: name}
-		ret[addr] = s.Removed(ctx, addr)
+		ret[addr] = s.RemovedComponent(addr)
 	}
 	return ret
 }
@@ -482,7 +451,7 @@ func (s *StackConfig) resolveExpressionReference(
 	// unit tests in this package. The function called in this branch will
 	// return an error itself if we're not running in a suitable test situation.
 	if addr, ok := ref.Target.(stackaddrs.TestOnlyGlobal); ok {
-		return s.main.resolveTestOnlyGlobalReference(ctx, addr, ref.SourceRange)
+		return s.main.resolveTestOnlyGlobalReference(addr, ref.SourceRange)
 	}
 
 	// TODO: Most of the below would benefit from "Did you mean..." suggestions
@@ -490,7 +459,7 @@ func (s *StackConfig) resolveExpressionReference(
 
 	switch addr := ref.Target.(type) {
 	case stackaddrs.InputVariable:
-		ret := s.InputVariable(ctx, addr)
+		ret := s.InputVariable(addr)
 		if ret == nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -502,7 +471,7 @@ func (s *StackConfig) resolveExpressionReference(
 		}
 		return ret, diags
 	case stackaddrs.LocalValue:
-		ret := s.LocalValue(ctx, addr)
+		ret := s.LocalValue(addr)
 		if ret == nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -514,7 +483,7 @@ func (s *StackConfig) resolveExpressionReference(
 		}
 		return ret, diags
 	case stackaddrs.Component:
-		ret := s.Component(ctx, addr)
+		ret := s.Component(addr)
 		if ret == nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -526,7 +495,7 @@ func (s *StackConfig) resolveExpressionReference(
 		}
 		return ret, diags
 	case stackaddrs.StackCall:
-		ret := s.StackCall(ctx, addr)
+		ret := s.StackCall(addr)
 		if ret == nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -538,7 +507,7 @@ func (s *StackConfig) resolveExpressionReference(
 		}
 		return ret, diags
 	case stackaddrs.ProviderConfigRef:
-		ret := s.ProviderByLocalAddr(ctx, addr)
+		ret := s.ProviderByLocalAddr(addr)
 		if ret == nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -624,28 +593,3 @@ func (s *StackConfig) ExternalFunctions(ctx context.Context) (lang.ExternalFuncs
 func (s *StackConfig) PlanTimestamp() time.Time {
 	return s.main.PlanTimestamp()
 }
-
-// reportNamedPromises implements namedPromiseReporter.
-func (s *StackConfig) reportNamedPromises(cb func(id promising.PromiseID, name string)) {
-	s.mu.Lock()
-	defer s.mu.Unlock()
-
-	for _, child := range s.children {
-		child.reportNamedPromises(cb)
-	}
-	for _, child := range s.inputVariables {
-		child.reportNamedPromises(cb)
-	}
-	for _, child := range s.outputValues {
-		child.reportNamedPromises(cb)
-	}
-	for _, child := range s.stackCalls {
-		child.reportNamedPromises(cb)
-	}
-	for _, child := range s.components {
-		child.reportNamedPromises(cb)
-	}
-	for _, child := range s.providers {
-		child.reportNamedPromises(cb)
-	}
-}
