@@ -175,7 +175,7 @@ terraform {}
 
 func TestContext_missingPlugins(t *testing.T) {
 	ctx, diags := NewContext(&ContextOpts{})
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	configSrc := `
 terraform {
@@ -282,7 +282,7 @@ resource "implicit_thing" "b" {
 					),
 				)
 			}
-			assertDiagnosticsMatch(t, gotDiags, wantDiags)
+			tfdiags.AssertDiagnosticsMatch(t, gotDiags, wantDiags)
 		})
 	}
 }
@@ -326,7 +326,7 @@ func TestContext_preloadedProviderSchemas(t *testing.T) {
 		`,
 	})
 	_, diags := tfCore.Schemas(cfg, states.NewState())
-	assertNoDiagnostics(t, diags)
+	tfdiags.AssertNoDiagnostics(t, diags)
 
 	if provider.GetProviderSchemaCalled {
 		t.Error("called GetProviderSchema even though a preloaded schema was provided")
@@ -1014,77 +1014,6 @@ func legacyDiffComparisonString(changes *plans.ChangesSrc) string {
 	return buf.String()
 }
 
-// assertNoDiagnostics fails the test in progress (using t.Fatal) if the given
-// diagnostics is non-empty.
-func assertNoDiagnostics(t *testing.T, diags tfdiags.Diagnostics) {
-	t.Helper()
-	if len(diags) == 0 {
-		return
-	}
-	logDiagnostics(t, diags)
-	t.FailNow()
-}
-
-// assertNoDiagnostics fails the test in progress (using t.Fatal) if the given
-// diagnostics has any errors.
-func assertNoErrors(t *testing.T, diags tfdiags.Diagnostics) {
-	t.Helper()
-	if !diags.HasErrors() {
-		return
-	}
-	logDiagnostics(t, diags)
-	t.FailNow()
-}
-
-// assertDiagnosticsMatch fails the test in progress (using t.Fatal) if the
-// two sets of diagnostics don't match after being normalized using the
-// "ForRPC" processing step, which eliminates the specific type information
-// and HCL expression information of each diagnostic.
-//
-// assertDiagnosticsMatch sorts the two sets of diagnostics in the usual way
-// before comparing them, though diagnostics only have a partial order so that
-// will not totally normalize the ordering of all diagnostics sets.
-func assertDiagnosticsMatch(t *testing.T, got, want tfdiags.Diagnostics) {
-	got = got.ForRPC()
-	want = want.ForRPC()
-	got.Sort()
-	want.Sort()
-	if diff := cmp.Diff(want, got); diff != "" {
-		t.Fatalf("wrong diagnostics\n%s", diff)
-	}
-}
-
-type SummaryAndDetail struct {
-	Severity    tfdiags.Severity
-	Subject     string
-	Description string
-}
-
-// assertDiagnosticsSummaryAndDetailMatch fails the test in progress (using t.Fatal) if the
-// two sets of diagnostics don't match their subjects / descriptions after being normalized using the "ForRPC" processing step.
-func assertDiagnosticsSummaryAndDetailMatch(t *testing.T, got, want tfdiags.Diagnostics) {
-	t.Helper()
-	got = got.ForRPC()
-	want = want.ForRPC()
-
-	got.Sort()
-	want.Sort()
-
-	gotSummaryAndDetail := make([]SummaryAndDetail, len(got))
-	for i, diag := range got {
-		gotSummaryAndDetail[i] = SummaryAndDetail{diag.Severity(), diag.Description().Summary, diag.Description().Detail}
-	}
-
-	wantSummaryAndDetail := make([]SummaryAndDetail, len(want))
-	for i, diag := range want {
-		wantSummaryAndDetail[i] = SummaryAndDetail{diag.Severity(), diag.Description().Summary, diag.Description().Detail}
-	}
-
-	if diff := cmp.Diff(wantSummaryAndDetail, gotSummaryAndDetail); diff != "" {
-		t.Fatalf("wrong diagnostics\n%s", diff)
-	}
-}
-
 // checkPlanCompleteAndApplyable reports testing errors if the plan is not
 // flagged as being both complete and applyable.
 //
@@ -1138,43 +1067,6 @@ func checkPlanErrored(t *testing.T, plan *plans.Plan) {
 	}
 }
 
-// logDiagnostics is a test helper that logs the given diagnostics to to the
-// given testing.T using t.Log, in a way that is hopefully useful in debugging
-// a test. It does not generate any errors or fail the test. See
-// assertNoDiagnostics and assertNoErrors for more specific helpers that can
-// also fail the test.
-func logDiagnostics(t *testing.T, diags tfdiags.Diagnostics) {
-	t.Helper()
-	for _, diag := range diags {
-		desc := diag.Description()
-		rng := diag.Source()
-
-		var severity string
-		switch diag.Severity() {
-		case tfdiags.Error:
-			severity = "ERROR"
-		case tfdiags.Warning:
-			severity = "WARN"
-		default:
-			severity = "???" // should never happen
-		}
-
-		if subj := rng.Subject; subj != nil {
-			if desc.Detail == "" {
-				t.Logf("[%s@%s] %s", severity, subj.StartString(), desc.Summary)
-			} else {
-				t.Logf("[%s@%s] %s: %s", severity, subj.StartString(), desc.Summary, desc.Detail)
-			}
-		} else {
-			if desc.Detail == "" {
-				t.Logf("[%s] %s", severity, desc.Summary)
-			} else {
-				t.Logf("[%s] %s: %s", severity, desc.Summary, desc.Detail)
-			}
-		}
-	}
-}
-
 const testContextRefreshModuleStr = `
 aws_instance.web: (tainted)
   ID = bar
