@@ -249,7 +249,7 @@ func staticValidateResourceReference(modCfg *configs.Config, addr addrs.Resource
 	}
 
 	providerFqn := modCfg.Module.ProviderForLocalConfig(cfg.ProviderConfigAddr())
-	schema, _, err := plugins.ResourceTypeSchema(providerFqn, addr.Mode, addr.Type)
+	schema, err := plugins.ResourceTypeSchema(providerFqn, addr.Mode, addr.Type)
 	if err != nil {
 		// Prior validation should've taken care of a schema lookup error,
 		// so we should never get here but we'll handle it here anyway for
@@ -262,7 +262,7 @@ func staticValidateResourceReference(modCfg *configs.Config, addr addrs.Resource
 		})
 	}
 
-	if schema == nil {
+	if schema.Body == nil {
 		// Prior validation should've taken care of a resource block with an
 		// unsupported type, so we should never get here but we'll handle it
 		// here anyway for robustness.
@@ -298,7 +298,7 @@ func staticValidateResourceReference(modCfg *configs.Config, addr addrs.Resource
 
 	// If we got this far then we'll try to validate the remaining traversal
 	// steps against our schema.
-	moreDiags := schema.StaticValidateTraversal(remain)
+	moreDiags := schema.Body.StaticValidateTraversal(remain)
 	diags = diags.Append(moreDiags)
 
 	return diags
