@@ -16,7 +16,7 @@ import (
 func mockProviderWithConfigSchema(schema *configschema.Block) *testing_provider.MockProvider {
 	return &testing_provider.MockProvider{
 		GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
-			Provider: providers.Schema{Block: schema},
+			Provider: providers.Schema{Body: schema},
 		},
 	}
 }
@@ -27,7 +27,7 @@ func mockProviderWithResourceTypeSchema(name string, schema *configschema.Block)
 	return &testing_provider.MockProvider{
 		GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 			Provider: providers.Schema{
-				Block: &configschema.Block{
+				Body: &configschema.Block{
 					Attributes: map[string]*configschema.Attribute{
 						"string": {
 							Type:     cty.String,
@@ -45,7 +45,7 @@ func mockProviderWithResourceTypeSchema(name string, schema *configschema.Block)
 				},
 			},
 			ResourceTypes: map[string]providers.Schema{
-				name: providers.Schema{Block: schema},
+				name: providers.Schema{Body: schema},
 			},
 		},
 	}
@@ -71,12 +71,12 @@ func mockProviderWithResourceTypeSchema(name string, schema *configschema.Block)
 func simpleMockProvider() *testing_provider.MockProvider {
 	return &testing_provider.MockProvider{
 		GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
-			Provider: providers.Schema{Block: simpleTestSchema()},
+			Provider: providers.Schema{Body: simpleTestSchema()},
 			ResourceTypes: map[string]providers.Schema{
-				"test_object": providers.Schema{Block: simpleTestSchema()},
+				"test_object": providers.Schema{Body: simpleTestSchema()},
 			},
 			DataSources: map[string]providers.Schema{
-				"test_object": providers.Schema{Block: simpleTestSchema()},
+				"test_object": providers.Schema{Body: simpleTestSchema()},
 			},
 		},
 	}
@@ -97,20 +97,20 @@ func getProviderSchema(p *testing_provider.MockProvider) *providerSchema {
 	resp := p.GetProviderSchemaResponse
 
 	schema := &providerSchema{
-		Provider:                   resp.Provider.Block,
-		ProviderMeta:               resp.ProviderMeta.Block,
+		Provider:                   resp.Provider.Body,
+		ProviderMeta:               resp.ProviderMeta.Body,
 		ResourceTypes:              map[string]*configschema.Block{},
 		DataSources:                map[string]*configschema.Block{},
 		ResourceTypeSchemaVersions: map[string]uint64{},
 	}
 
 	for resType, s := range resp.ResourceTypes {
-		schema.ResourceTypes[resType] = s.Block
+		schema.ResourceTypes[resType] = s.Body
 		schema.ResourceTypeSchemaVersions[resType] = uint64(s.Version)
 	}
 
 	for dataSource, s := range resp.DataSources {
-		schema.DataSources[dataSource] = s.Block
+		schema.DataSources[dataSource] = s.Body
 	}
 
 	return schema
@@ -124,27 +124,37 @@ type providerSchema struct {
 	ResourceTypes              map[string]*configschema.Block
 	ResourceTypeSchemaVersions map[string]uint64
 	DataSources                map[string]*configschema.Block
+	IdentityTypes              map[string]*configschema.Object
+	IdentityTypeSchemaVersions map[string]uint64
 }
 
 // getProviderSchemaResponseFromProviderSchema is a test helper to convert a
 // providerSchema to a GetProviderSchemaResponse for use when building a mock provider.
 func getProviderSchemaResponseFromProviderSchema(providerSchema *providerSchema) *providers.GetProviderSchemaResponse {
 	resp := &providers.GetProviderSchemaResponse{
-		Provider:      providers.Schema{Block: providerSchema.Provider},
-		ProviderMeta:  providers.Schema{Block: providerSchema.ProviderMeta},
+		Provider:      providers.Schema{Body: providerSchema.Provider},
+		ProviderMeta:  providers.Schema{Body: providerSchema.ProviderMeta},
 		ResourceTypes: map[string]providers.Schema{},
 		DataSources:   map[string]providers.Schema{},
 	}
 
 	for name, schema := range providerSchema.ResourceTypes {
-		resp.ResourceTypes[name] = providers.Schema{
-			Block:   schema,
+		ps := providers.Schema{
+			Body:    schema,
 			Version: int64(providerSchema.ResourceTypeSchemaVersions[name]),
 		}
+
+		id, ok := providerSchema.IdentityTypes[name]
+		if ok {
+			ps.Identity = id
+			ps.IdentityVersion = int64(providerSchema.IdentityTypeSchemaVersions[name])
+		}
+
+		resp.ResourceTypes[name] = ps
 	}
 
 	for name, schema := range providerSchema.DataSources {
-		resp.DataSources[name] = providers.Schema{Block: schema}
+		resp.DataSources[name] = providers.Schema{Body: schema}
 	}
 
 	return resp
